@@ -22,6 +22,7 @@ use std::any::Any;
 use bytes::Bytes;
 use datafusion_common::not_impl_err;
 use datafusion_common::DataFusionError;
+use datafusion_common::FileType;
 use datafusion_execution::TaskContext;
 use rand::distributions::Alphanumeric;
 use rand::distributions::DistString;
@@ -40,7 +41,7 @@ use async_trait::async_trait;
 use bytes::Buf;
 
 use datafusion_physical_expr::PhysicalExpr;
-use object_store::{GetResult, ObjectMeta, ObjectStore};
+use object_store::{GetResultPayload, ObjectMeta, ObjectStore};
 
 use crate::datasource::physical_plan::FileGroupDisplay;
 use crate::physical_plan::insert::DataSink;
@@ -50,6 +51,7 @@ use crate::physical_plan::{DisplayAs, DisplayFormatType, Statistics};
 
 use super::FileFormat;
 use super::FileScanConfig;
+use crate::datasource::file_format::file_compression_type::FileCompressionType;
 use crate::datasource::file_format::write::{
     create_writer, stateless_serialize_and_write_files, BatchSerializer, FileWriterMode,
 };
@@ -59,7 +61,6 @@ use crate::datasource::physical_plan::NdJsonExec;
 use crate::error::Result;
 use crate::execution::context::SessionState;
 use crate::physical_plan::ExecutionPlan;
-use datafusion_common::FileCompressionType;
 
 /// New line delimited JSON `FileFormat` implementation.
 #[derive(Debug)]
@@ -120,14 +121,15 @@ impl FileFormat for JsonFormat {
                 should_take
             };
 
-            let schema = match store.get(&object.location).await? {
-                GetResult::File(file, _) => {
+            let r = store.as_ref().get(&object.location).await?;
+            let schema = match r.payload {
+                GetResultPayload::File(file, _) => {
                     let decoder = file_compression_type.convert_read(file)?;
                     let mut reader = BufReader::new(decoder);
                     let iter = ValueIter::new(&mut reader, None);
                     infer_json_schema_from_iterator(iter.take_while(|_| take_while()))?
                 }
-                r @ GetResult::Stream(_) => {
+                GetResultPayload::Stream(_) => {
                     let data = r.bytes().await?;
                     let decoder = file_compression_type.convert_read(data.reader())?;
                     let mut reader = BufReader::new(decoder);
@@ -184,6 +186,10 @@ impl FileFormat for JsonFormat {
 
         Ok(Arc::new(FileSinkExec::new(input, sink, sink_schema)) as _)
     }
+
+    fn file_type(&self) -> FileType {
+        FileType::JSON
+    }
 }
 
 impl Default for JsonSerializer {
@@ -215,6 +221,10 @@ impl BatchSerializer for JsonSerializer {
         //drop(writer);
         Ok(Bytes::from(self.buffer.drain(..).collect::<Vec<u8>>()))
     }
+
+    fn duplicate(&mut self) -> Result<Box<dyn BatchSerializer>> {
+        Ok(Box::new(JsonSerializer::new()))
+    }
 }
 
 /// Implements [`DataSink`] for writing to a Json file.
@@ -270,13 +280,17 @@ impl DataSink for JsonSink {
             .runtime_env()
             .object_store(&self.config.object_store_url)?;
 
+        let writer_options = self.config.file_type_writer_options.try_into_json()?;
+
+        let compression = FileCompressionType::from(writer_options.compression);
+
         // Construct serializer and writer for each file group
         let mut serializers: Vec<Box<dyn BatchSerializer>> = vec![];
         let mut writers = vec![];
         match self.config.writer_mode {
             FileWriterMode::Append => {
-                if !self.config.per_thread_output {
-                    return not_impl_err!("per_thread_output=false is not implemented for JsonSink in Append mode");
+                if self.config.single_file_output {
+                    return Err(DataFusionError::NotImplemented("single_file_output=true is not implemented for JsonSink in Append mode".into()));
                 }
                 for file_group in &self.config.file_groups {
                     let serializer = JsonSerializer::new();
@@ -285,7 +299,7 @@ impl DataSink for JsonSink {
                     let file = file_group.clone();
                     let writer = create_writer(
                         self.config.writer_mode,
-                        self.file_compression_type,
+                        compression,
                         file.object_meta.clone().into(),
                         object_store.clone(),
                     )
@@ -299,8 +313,8 @@ impl DataSink for JsonSink {
             FileWriterMode::PutMultipart => {
                 // Currently assuming only 1 partition path (i.e. not hive-style partitioning on a column)
                 let base_path = &self.config.table_paths[0];
-                match self.config.per_thread_output {
-                    true => {
+                match self.config.single_file_output {
+                    false => {
                         // Uniquely identify this batch of files with a random string, to prevent collisions overwriting files
                         let write_id =
                             Alphanumeric.sample_string(&mut rand::thread_rng(), 16);
@@ -318,7 +332,7 @@ impl DataSink for JsonSink {
                             };
                             let writer = create_writer(
                                 self.config.writer_mode,
-                                self.file_compression_type,
+                                compression,
                                 object_meta.into(),
                                 object_store.clone(),
                             )
@@ -326,7 +340,7 @@ impl DataSink for JsonSink {
                             writers.push(writer);
                         }
                     }
-                    false => {
+                    true => {
                         let serializer = JsonSerializer::new();
                         serializers.push(Box::new(serializer));
                         let file_path = base_path.prefix();
@@ -338,7 +352,7 @@ impl DataSink for JsonSink {
                         };
                         let writer = create_writer(
                             self.config.writer_mode,
-                            self.file_compression_type,
+                            compression,
                             object_meta.into(),
                             object_store.clone(),
                         )
@@ -353,7 +367,8 @@ impl DataSink for JsonSink {
             data,
             serializers,
             writers,
-            self.config.per_thread_output,
+            self.config.single_file_output,
+            self.config.unbounded_input,
         )
         .await
     }
@@ -374,7 +389,7 @@ mod tests {
     #[tokio::test]
     async fn read_small_batches() -> Result<()> {
         let config = SessionConfig::new().with_batch_size(2);
-        let session_ctx = SessionContext::with_config(config);
+        let session_ctx = SessionContext::new_with_config(config);
         let state = session_ctx.state();
         let task_ctx = state.task_ctx();
         let projection = None;
