@@ -24,6 +24,7 @@ pub const DEFAULT_SCHEMA_INFER_MAX_RECORD: usize = 1000;
 pub mod arrow;
 pub mod avro;
 pub mod csv;
+pub mod file_compression_type;
 pub mod json;
 pub mod options;
 pub mod parquet;
@@ -39,7 +40,7 @@ use crate::error::Result;
 use crate::execution::context::SessionState;
 use crate::physical_plan::{ExecutionPlan, Statistics};
 
-use datafusion_common::{not_impl_err, DataFusionError};
+use datafusion_common::{not_impl_err, DataFusionError, FileType};
 use datafusion_physical_expr::PhysicalExpr;
 
 use async_trait::async_trait;
@@ -101,6 +102,9 @@ pub trait FileFormat: Send + Sync + fmt::Debug {
     ) -> Result<Arc<dyn ExecutionPlan>> {
         not_impl_err!("Writer not implemented for this format")
     }
+
+    /// Returns the FileType corresponding to this FileFormat
+    fn file_type(&self) -> FileType;
 }
 
 #[cfg(test)]
@@ -117,7 +121,9 @@ pub(crate) mod test_util {
     use futures::StreamExt;
     use object_store::local::LocalFileSystem;
     use object_store::path::Path;
-    use object_store::{GetOptions, GetResult, ListResult, MultipartId};
+    use object_store::{
+        GetOptions, GetResult, GetResultPayload, ListResult, MultipartId,
+    };
     use tokio::io::AsyncWrite;
 
     pub async fn scan_format(
@@ -201,18 +207,28 @@ pub(crate) mod test_util {
             unimplemented!()
         }
 
-        async fn get(&self, _location: &Path) -> object_store::Result<GetResult> {
+        async fn get(&self, location: &Path) -> object_store::Result<GetResult> {
             let bytes = self.bytes_to_repeat.clone();
+            let range = 0..bytes.len() * self.max_iterations;
             let arc = self.iterations_detected.clone();
-            Ok(GetResult::Stream(
-                futures::stream::repeat_with(move || {
-                    let arc_inner = arc.clone();
-                    *arc_inner.lock().unwrap() += 1;
-                    Ok(bytes.clone())
-                })
-                .take(self.max_iterations)
-                .boxed(),
-            ))
+            let stream = futures::stream::repeat_with(move || {
+                let arc_inner = arc.clone();
+                *arc_inner.lock().unwrap() += 1;
+                Ok(bytes.clone())
+            })
+            .take(self.max_iterations)
+            .boxed();
+
+            Ok(GetResult {
+                payload: GetResultPayload::Stream(stream),
+                meta: ObjectMeta {
+                    location: location.clone(),
+                    last_modified: Default::default(),
+                    size: range.end,
+                    e_tag: None,
+                },
+                range: Default::default(),
+            })
         }
 
         async fn get_opts(
