@@ -17,18 +17,24 @@
 
 //! Parquet format abstractions
 
-use parquet::basic::{BrotliLevel, GzipLevel, ZstdLevel};
+use parquet::column::writer::ColumnCloseResult;
+use parquet::file::writer::SerializedFileWriter;
 use rand::distributions::DistString;
 use std::any::Any;
 use std::fmt;
 use std::fmt::Debug;
+use std::io::Write;
 use std::sync::Arc;
+use tokio::io::{AsyncWrite, AsyncWriteExt};
+use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
+use tokio::task::{JoinHandle, JoinSet};
 
+use crate::datasource::file_format::file_compression_type::FileCompressionType;
 use arrow::datatypes::SchemaRef;
 use arrow::datatypes::{Fields, Schema};
 use async_trait::async_trait;
 use bytes::{BufMut, BytesMut};
-use datafusion_common::{not_impl_err, plan_err, DataFusionError};
+use datafusion_common::{exec_err, not_impl_err, plan_err, DataFusionError, FileType};
 use datafusion_execution::TaskContext;
 use datafusion_physical_expr::PhysicalExpr;
 use futures::{StreamExt, TryStreamExt};
@@ -37,11 +43,11 @@ use object_store::{ObjectMeta, ObjectStore};
 use parquet::arrow::{parquet_to_arrow_schema, AsyncArrowWriter};
 use parquet::file::footer::{decode_footer, decode_metadata};
 use parquet::file::metadata::ParquetMetaData;
-use parquet::file::properties::{EnabledStatistics, WriterProperties, WriterVersion};
+use parquet::file::properties::WriterProperties;
 use parquet::file::statistics::Statistics as ParquetStatistics;
 use rand::distributions::Alphanumeric;
 
-use super::write::FileWriterMode;
+use super::write::{create_writer, AbortableWrite, FileWriterMode};
 use super::FileFormat;
 use super::FileScanConfig;
 use crate::arrow::array::{
@@ -64,9 +70,6 @@ use crate::physical_plan::{
     Statistics,
 };
 
-/// The number of files to read in parallel when inferring schema
-const SCHEMA_INFERENCE_CONCURRENCY: usize = 32;
-
 /// The Apache Parquet `FileFormat` implementation
 ///
 /// Note it is recommended these are instead configured on the [`ConfigOptions`]
@@ -170,7 +173,7 @@ impl FileFormat for ParquetFormat {
         let schemas: Vec<_> = futures::stream::iter(objects)
             .map(|object| fetch_schema(store.as_ref(), object, self.metadata_size_hint))
             .boxed() // Workaround https://github.com/rust-lang/rust/issues/64552
-            .buffered(SCHEMA_INFERENCE_CONCURRENCY)
+            .buffered(state.config_options().execution.meta_fetch_concurrency)
             .try_collect()
             .await?;
 
@@ -236,6 +239,10 @@ impl FileFormat for ParquetFormat {
 
         Ok(Arc::new(FileSinkExec::new(input, sink, sink_schema)) as _)
     }
+
+    fn file_type(&self) -> FileType {
+        FileType::PARQUET
+    }
 }
 
 fn summarize_min_max(
@@ -414,10 +421,7 @@ pub async fn fetch_parquet_metadata(
     size_hint: Option<usize>,
 ) -> Result<ParquetMetaData> {
     if meta.size < 8 {
-        return Err(DataFusionError::Execution(format!(
-            "file size of {} is less than footer",
-            meta.size
-        )));
+        return exec_err!("file size of {} is less than footer", meta.size);
     }
 
     // If a size hint is provided, read more than the minimum size
@@ -440,11 +444,11 @@ pub async fn fetch_parquet_metadata(
     let length = decode_footer(&footer)?;
 
     if meta.size < length + 8 {
-        return Err(DataFusionError::Execution(format!(
+        return exec_err!(
             "file size of {} is less than footer + metadata {}",
             meta.size,
             length + 8
-        )));
+        );
     }
 
     // Did not fetch the entire file metadata in the initial read, need to make a second request
@@ -599,214 +603,14 @@ impl DisplayAs for ParquetSink {
     }
 }
 
-/// Parses datafusion.execution.parquet.encoding String to a parquet::basic::Encoding
-fn parse_encoding_string(str_setting: &str) -> Result<parquet::basic::Encoding> {
-    let str_setting_lower: &str = &str_setting.to_lowercase();
-    match str_setting_lower {
-        "plain" => Ok(parquet::basic::Encoding::PLAIN),
-        "plain_dictionary" => Ok(parquet::basic::Encoding::PLAIN_DICTIONARY),
-        "rle" => Ok(parquet::basic::Encoding::RLE),
-        "bit_packed" => Ok(parquet::basic::Encoding::BIT_PACKED),
-        "delta_binary_packed" => Ok(parquet::basic::Encoding::DELTA_BINARY_PACKED),
-        "delta_length_byte_array" => {
-            Ok(parquet::basic::Encoding::DELTA_LENGTH_BYTE_ARRAY)
-        }
-        "delta_byte_array" => Ok(parquet::basic::Encoding::DELTA_BYTE_ARRAY),
-        "rle_dictionary" => Ok(parquet::basic::Encoding::RLE_DICTIONARY),
-        "byte_stream_split" => Ok(parquet::basic::Encoding::BYTE_STREAM_SPLIT),
-        _ => Err(DataFusionError::Plan(format!(
-            "Unknown or unsupported parquet encoding: \
-        {str_setting}. Valid values are: plain, plain_dictionary, rle, \
-        /// bit_packed, delta_binary_packed, delta_length_byte_array, \
-        /// delta_byte_array, rle_dictionary, and byte_stream_split."
-        ))),
-    }
-}
-
-/// Splits compression string into compression codec and optional compression_level
-/// I.e. gzip(2) -> gzip, 2
-fn split_compression_string(str_setting: &str) -> Result<(&str, Option<u32>)> {
-    let split_setting = str_setting.split_once('(');
-
-    match split_setting {
-        Some((codec, rh)) => {
-            let level = &rh[..rh.len() - 1].parse::<u32>().map_err(|_| {
-                DataFusionError::Plan(format!(
-                    "Could not parse compression string. \
-                    Got codec: {} and unknown level from {}",
-                    codec, str_setting
-                ))
-            })?;
-            Ok((codec, Some(*level)))
-        }
-        None => Ok((str_setting, None)),
-    }
-}
-
-/// Helper to ensure compression codecs which don't support levels
-/// don't have one set. E.g. snappy(2) is invalid.
-fn check_level_is_none(codec: &str, level: &Option<u32>) -> Result<()> {
-    if level.is_some() {
-        return Err(DataFusionError::Plan(format!(
-            "Compression {codec} does not support specifying a level"
-        )));
-    }
-    Ok(())
-}
-
-/// Helper to ensure compression codecs which require a level
-/// do have one set. E.g. zstd is invalid, zstd(3) is valid
-fn require_level(codec: &str, level: Option<u32>) -> Result<u32> {
-    level.ok_or(DataFusionError::Plan(format!(
-        "{codec} compression requires specifying a level such as {codec}(4)"
-    )))
-}
-
-/// Parses datafusion.execution.parquet.compression String to a parquet::basic::Compression
-fn parse_compression_string(str_setting: &str) -> Result<parquet::basic::Compression> {
-    let str_setting_lower: &str = &str_setting.to_lowercase();
-    let (codec, level) = split_compression_string(str_setting_lower)?;
-    match codec {
-        "uncompressed" => {
-            check_level_is_none(codec, &level)?;
-            Ok(parquet::basic::Compression::UNCOMPRESSED)
-        }
-        "snappy" => {
-            check_level_is_none(codec, &level)?;
-            Ok(parquet::basic::Compression::SNAPPY)
-        }
-        "gzip" => {
-            let level = require_level(codec, level)?;
-            Ok(parquet::basic::Compression::GZIP(GzipLevel::try_new(
-                level,
-            )?))
-        }
-        "lzo" => {
-            check_level_is_none(codec, &level)?;
-            Ok(parquet::basic::Compression::LZO)
-        }
-        "brotli" => {
-            let level = require_level(codec, level)?;
-            Ok(parquet::basic::Compression::BROTLI(BrotliLevel::try_new(
-                level,
-            )?))
-        }
-        "lz4" => {
-            check_level_is_none(codec, &level)?;
-            Ok(parquet::basic::Compression::LZ4)
-        }
-        "zstd" => {
-            let level = require_level(codec, level)?;
-            Ok(parquet::basic::Compression::ZSTD(ZstdLevel::try_new(
-                level as i32,
-            )?))
-        }
-        "lz4_raw" => {
-            check_level_is_none(codec, &level)?;
-            Ok(parquet::basic::Compression::LZ4_RAW)
-        }
-        _ => Err(DataFusionError::Plan(format!(
-            "Unknown or unsupported parquet compression: \
-        {str_setting}. Valid values are: uncompressed, snappy, gzip(level), \
-        lzo, brotli(level), lz4, zstd(level), and lz4_raw."
-        ))),
-    }
-}
-
-fn parse_version_string(str_setting: &str) -> Result<WriterVersion> {
-    let str_setting_lower: &str = &str_setting.to_lowercase();
-    match str_setting_lower {
-        "1.0" => Ok(WriterVersion::PARQUET_1_0),
-        "2.0" => Ok(WriterVersion::PARQUET_2_0),
-        _ => Err(DataFusionError::Plan(format!(
-            "Unknown or unsupported parquet writer version {str_setting} \
-            valid options are '1.0' and '2.0'"
-        ))),
-    }
-}
-
-fn parse_statistics_string(str_setting: &str) -> Result<EnabledStatistics> {
-    let str_setting_lower: &str = &str_setting.to_lowercase();
-    match str_setting_lower {
-        "none" => Ok(EnabledStatistics::None),
-        "chunk" => Ok(EnabledStatistics::Chunk),
-        "page" => Ok(EnabledStatistics::Page),
-        _ => Err(DataFusionError::Plan(format!(
-            "Unknown or unsupported parquet statistics setting {str_setting} \
-            valid options are 'none', 'page', and 'chunk'"
-        ))),
-    }
-}
-
 impl ParquetSink {
     fn new(config: FileSinkConfig) -> Self {
         Self { config }
     }
 
-    /// Builds a parquet WriterProperties struct, setting options as appropriate from TaskContext options.
-    /// May return error if SessionContext contains invalid or unsupported options
-    fn parquet_writer_props_from_context(
-        &self,
-        context: &Arc<TaskContext>,
-    ) -> Result<WriterProperties> {
-        let parquet_context = &context.session_config().options().execution.parquet;
-        let mut builder = WriterProperties::builder()
-            .set_data_page_size_limit(parquet_context.data_pagesize_limit)
-            .set_write_batch_size(parquet_context.write_batch_size)
-            .set_writer_version(parse_version_string(&parquet_context.writer_version)?)
-            .set_dictionary_page_size_limit(parquet_context.dictionary_page_size_limit)
-            .set_max_row_group_size(parquet_context.max_row_group_size)
-            .set_created_by(parquet_context.created_by.clone())
-            .set_column_index_truncate_length(
-                parquet_context.column_index_truncate_length,
-            )
-            .set_data_page_row_count_limit(parquet_context.data_page_row_count_limit)
-            .set_bloom_filter_enabled(parquet_context.bloom_filter_enabled);
-
-        builder = match &parquet_context.encoding {
-            Some(encoding) => builder.set_encoding(parse_encoding_string(encoding)?),
-            None => builder,
-        };
-
-        builder = match &parquet_context.dictionary_enabled {
-            Some(enabled) => builder.set_dictionary_enabled(*enabled),
-            None => builder,
-        };
-
-        builder = match &parquet_context.compression {
-            Some(compression) => {
-                builder.set_compression(parse_compression_string(compression)?)
-            }
-            None => builder,
-        };
-
-        builder = match &parquet_context.statistics_enabled {
-            Some(statistics) => {
-                builder.set_statistics_enabled(parse_statistics_string(statistics)?)
-            }
-            None => builder,
-        };
-
-        builder = match &parquet_context.max_statistics_size {
-            Some(size) => builder.set_max_statistics_size(*size),
-            None => builder,
-        };
-
-        builder = match &parquet_context.bloom_filter_fpp {
-            Some(fpp) => builder.set_bloom_filter_fpp(*fpp),
-            None => builder,
-        };
-
-        builder = match &parquet_context.bloom_filter_ndv {
-            Some(ndv) => builder.set_bloom_filter_ndv(*ndv),
-            None => builder,
-        };
-
-        Ok(builder.build())
-    }
-
-    // Create a write for parquet files
-    async fn create_writer(
+    /// Creates an AsyncArrowWriter which serializes a parquet file to an ObjectStore
+    /// AsyncArrowWriters are used when individual parquet file serialization is not parallelized
+    async fn create_async_arrow_writer(
         &self,
         file_meta: FileMeta,
         object_store: Arc<dyn ObjectStore>,
@@ -839,22 +643,17 @@ impl ParquetSink {
             }
         }
     }
-}
 
-#[async_trait]
-impl DataSink for ParquetSink {
-    async fn write_all(
+    /// Creates an AsyncArrowWriter for each partition to be written out
+    /// AsyncArrowWriters are used when individual parquet file serialization is not parallelized
+    async fn create_all_async_arrow_writers(
         &self,
-        mut data: Vec<SendableRecordBatchStream>,
-        context: &Arc<TaskContext>,
-    ) -> Result<u64> {
-        let num_partitions = data.len();
-        let parquet_props = self.parquet_writer_props_from_context(context)?;
-
-        let object_store = context
-            .runtime_env()
-            .object_store(&self.config.object_store_url)?;
-
+        num_partitions: usize,
+        parquet_props: &WriterProperties,
+        object_store: Arc<dyn ObjectStore>,
+    ) -> Result<
+        Vec<AsyncArrowWriter<Box<dyn tokio::io::AsyncWrite + std::marker::Send + Unpin>>>,
+    > {
         // Construct writer for each file group
         let mut writers = vec![];
         match self.config.writer_mode {
@@ -869,8 +668,8 @@ impl DataSink for ParquetSink {
             FileWriterMode::PutMultipart => {
                 // Currently assuming only 1 partition path (i.e. not hive-style partitioning on a column)
                 let base_path = &self.config.table_paths[0];
-                match self.config.per_thread_output {
-                    true => {
+                match self.config.single_file_output {
+                    false => {
                         // Uniquely identify this batch of files with a random string, to prevent collisions overwriting files
                         let write_id =
                             Alphanumeric.sample_string(&mut rand::thread_rng(), 16);
@@ -885,7 +684,7 @@ impl DataSink for ParquetSink {
                                 e_tag: None,
                             };
                             let writer = self
-                                .create_writer(
+                                .create_async_arrow_writer(
                                     object_meta.into(),
                                     object_store.clone(),
                                     parquet_props.clone(),
@@ -894,7 +693,7 @@ impl DataSink for ParquetSink {
                             writers.push(writer);
                         }
                     }
-                    false => {
+                    true => {
                         let file_path = base_path.prefix();
                         let object_meta = ObjectMeta {
                             location: file_path.clone(),
@@ -903,7 +702,7 @@ impl DataSink for ParquetSink {
                             e_tag: None,
                         };
                         let writer = self
-                            .create_writer(
+                            .create_async_arrow_writer(
                                 object_meta.into(),
                                 object_store.clone(),
                                 parquet_props.clone(),
@@ -915,29 +714,338 @@ impl DataSink for ParquetSink {
             }
         }
 
-        let mut row_count = 0;
-        // TODO parallelize serialization accross partitions and batches within partitions
-        // see: https://github.com/apache/arrow-datafusion/issues/7079
-        for (part_idx, data_stream) in data.iter_mut().enumerate().take(num_partitions) {
-            let idx = match self.config.per_thread_output {
-                true => part_idx,
-                false => 0,
+        Ok(writers)
+    }
+
+    /// Creates an object store writer for each output partition
+    /// This is used when parallelizing individual parquet file writes.
+    async fn create_object_store_writers(
+        &self,
+        num_partitions: usize,
+        object_store: Arc<dyn ObjectStore>,
+    ) -> Result<Vec<AbortableWrite<Box<dyn AsyncWrite + Send + Unpin>>>> {
+        let mut writers = Vec::new();
+
+        for _ in 0..num_partitions {
+            let file_path = self.config.table_paths[0].prefix();
+            let object_meta = ObjectMeta {
+                location: file_path.clone(),
+                last_modified: chrono::offset::Utc::now(),
+                size: 0,
+                e_tag: None,
             };
-            while let Some(batch) = data_stream.next().await.transpose()? {
-                row_count += batch.num_rows();
-                // TODO cleanup all multipart writes when any encounters an error
-                writers[idx].write(&batch).await?;
+            writers.push(
+                create_writer(
+                    FileWriterMode::PutMultipart,
+                    FileCompressionType::UNCOMPRESSED,
+                    object_meta.into(),
+                    object_store.clone(),
+                )
+                .await?,
+            );
+        }
+
+        Ok(writers)
+    }
+}
+
+#[async_trait]
+impl DataSink for ParquetSink {
+    async fn write_all(
+        &self,
+        mut data: Vec<SendableRecordBatchStream>,
+        context: &Arc<TaskContext>,
+    ) -> Result<u64> {
+        let num_partitions = data.len();
+        let parquet_props = self
+            .config
+            .file_type_writer_options
+            .try_into_parquet()?
+            .writer_options();
+
+        let object_store = context
+            .runtime_env()
+            .object_store(&self.config.object_store_url)?;
+
+        let mut row_count = 0;
+
+        let allow_single_file_parallelism = context
+            .session_config()
+            .options()
+            .execution
+            .parquet
+            .allow_single_file_parallelism;
+
+        match self.config.single_file_output {
+            false => {
+                let writers = self
+                    .create_all_async_arrow_writers(
+                        num_partitions,
+                        parquet_props,
+                        object_store.clone(),
+                    )
+                    .await?;
+                // TODO parallelize individual parquet serialization when already outputting multiple parquet files
+                // e.g. if outputting 2 parquet files on a system with 32 threads, spawn 16 tasks for each individual
+                // file to be serialized.
+                row_count = output_multiple_parquet_files(writers, data).await?;
+            }
+            true => {
+                if !allow_single_file_parallelism || data.len() <= 1 {
+                    let mut writer = self
+                        .create_all_async_arrow_writers(
+                            num_partitions,
+                            parquet_props,
+                            object_store.clone(),
+                        )
+                        .await?
+                        .remove(0);
+                    for data_stream in data.iter_mut() {
+                        while let Some(batch) = data_stream.next().await.transpose()? {
+                            row_count += batch.num_rows();
+                            writer.write(&batch).await?;
+                        }
+                    }
+
+                    writer.close().await?;
+                } else {
+                    let object_store_writer = self
+                        .create_object_store_writers(1, object_store)
+                        .await?
+                        .remove(0);
+                    row_count = output_single_parquet_file_parallelized(
+                        object_store_writer,
+                        data,
+                        self.config.output_schema.clone(),
+                        parquet_props,
+                    )
+                    .await?;
+                }
             }
         }
 
-        for writer in writers {
-            writer.close().await?;
-        }
-
         Ok(row_count as u64)
     }
 }
 
+/// This is the return type when joining subtasks which are serializing parquet files
+/// into memory buffers. The first part of the tuple is the parquet bytes and the
+/// second is how many rows were written into the file.
+type ParquetFileSerializedResult = Result<(Vec<u8>, usize), DataFusionError>;
+
+/// Parallelizes the serialization of a single parquet file, by first serializing N
+/// independent RecordBatch streams in parallel to parquet files in memory. Another
+/// task then stitches these independent files back together and streams this large
+/// single parquet file to an ObjectStore in multiple parts.
+async fn output_single_parquet_file_parallelized(
+    mut object_store_writer: AbortableWrite<Box<dyn AsyncWrite + Send + Unpin>>,
+    mut data: Vec<SendableRecordBatchStream>,
+    output_schema: Arc<Schema>,
+    parquet_props: &WriterProperties,
+) -> Result<usize> {
+    let mut row_count = 0;
+    // TODO decrease parallelism / buffering:
+    // https://github.com/apache/arrow-datafusion/issues/7591
+    let parallelism = data.len();
+    let mut join_handles: Vec<JoinHandle<ParquetFileSerializedResult>> =
+        Vec::with_capacity(parallelism);
+    for _ in 0..parallelism {
+        let buffer: Vec<u8> = Vec::new();
+        let mut writer = parquet::arrow::arrow_writer::ArrowWriter::try_new(
+            buffer,
+            output_schema.clone(),
+            Some(parquet_props.clone()),
+        )?;
+        let mut data_stream = data.remove(0);
+        join_handles.push(tokio::spawn(async move {
+            let mut inner_row_count = 0;
+            while let Some(batch) = data_stream.next().await.transpose()? {
+                inner_row_count += batch.num_rows();
+                writer.write(&batch)?;
+            }
+            let out = writer.into_inner()?;
+            Ok((out, inner_row_count))
+        }))
+    }
+
+    let mut writer = None;
+    let endpoints: (UnboundedSender<Vec<u8>>, UnboundedReceiver<Vec<u8>>) =
+        tokio::sync::mpsc::unbounded_channel();
+    let (tx, mut rx) = endpoints;
+    let writer_join_handle: JoinHandle<
+        Result<
+            AbortableWrite<Box<dyn tokio::io::AsyncWrite + std::marker::Send + Unpin>>,
+            DataFusionError,
+        >,
+    > = tokio::task::spawn(async move {
+        while let Some(data) = rx.recv().await {
+            // TODO write incrementally
+            // https://github.com/apache/arrow-datafusion/issues/7591
+            object_store_writer.write_all(data.as_slice()).await?;
+        }
+        Ok(object_store_writer)
+    });
+    let merged_buff = SharedBuffer::new(1048576);
+    for handle in join_handles {
+        let join_result = handle.await;
+        match join_result {
+            Ok(result) => {
+                let (out, num_rows) = result?;
+                let reader = bytes::Bytes::from(out);
+                row_count += num_rows;
+                //let reader = File::open(buffer)?;
+                let metadata = parquet::file::footer::parse_metadata(&reader)?;
+                let schema = metadata.file_metadata().schema();
+                writer = match writer {
+                    Some(writer) => Some(writer),
+                    None => Some(SerializedFileWriter::new(
+                        merged_buff.clone(),
+                        Arc::new(schema.clone()),
+                        Arc::new(parquet_props.clone()),
+                    )?),
+                };
+
+                match &mut writer{
+                    Some(w) => {
+                        // Note: cannot use .await within this loop as RowGroupMetaData is not Send
+                        // Instead, use a non-blocking channel to send bytes to separate worker
+                        // which will write to ObjectStore.
+                        for rg in metadata.row_groups() {
+                            let mut rg_out = w.next_row_group()?;
+                            for column in rg.columns() {
+                                let result = ColumnCloseResult {
+                                    bytes_written: column.compressed_size() as _,
+                                    rows_written: rg.num_rows() as _,
+                                    metadata: column.clone(),
+                                    // TODO need to populate the indexes when writing final file
+                                    // see https://github.com/apache/arrow-datafusion/issues/7589
+                                    bloom_filter: None,
+                                    column_index: None,
+                                    offset_index: None,
+                                };
+                                rg_out.append_column(&reader, result)?;
+                                let mut buff_to_flush = merged_buff.buffer.try_lock().unwrap();
+                                if buff_to_flush.len() > 1024000{
+                                    let bytes: Vec<u8> = buff_to_flush.drain(..).collect();
+                                    tx.send(bytes).map_err(|_| DataFusionError::Execution("Failed to send bytes to ObjectStore writer".into()))?;
+
+                                }
+                            }
+                            rg_out.close()?;
+                            let mut buff_to_flush = merged_buff.buffer.try_lock().unwrap();
+                            if buff_to_flush.len() > 1024000{
+                                let bytes: Vec<u8> = buff_to_flush.drain(..).collect();
+                                tx.send(bytes).map_err(|_| DataFusionError::Execution("Failed to send bytes to ObjectStore writer".into()))?;
+                            }
+                        }
+                    },
+                    None => unreachable!("Parquet writer should always be initialized in first iteration of loop!")
+                }
+            }
+            Err(e) => {
+                if e.is_panic() {
+                    std::panic::resume_unwind(e.into_panic());
+                } else {
+                    unreachable!();
+                }
+            }
+        }
+    }
+    let inner_writer = writer.unwrap().into_inner()?;
+    let final_buff = inner_writer.buffer.try_lock().unwrap();
+
+    // Explicitly drop tx to signal to rx we are done sending data
+    drop(tx);
+
+    let mut object_store_writer = match writer_join_handle.await {
+        Ok(r) => r?,
+        Err(e) => {
+            if e.is_panic() {
+                std::panic::resume_unwind(e.into_panic())
+            } else {
+                unreachable!()
+            }
+        }
+    };
+    object_store_writer.write_all(final_buff.as_slice()).await?;
+    object_store_writer.shutdown().await?;
+    println!("done!");
+
+    Ok(row_count)
+}
+
+/// Serializes multiple parquet files independently in parallel from different RecordBatch streams.
+/// AsyncArrowWriter is used to coordinate serialization and MultiPart puts to ObjectStore
+/// Only a single CPU thread is used to serialize each individual parquet file, so write speed and overall
+/// CPU utilization is dependent on the number of output files.
+async fn output_multiple_parquet_files(
+    writers: Vec<
+        AsyncArrowWriter<Box<dyn tokio::io::AsyncWrite + std::marker::Send + Unpin>>,
+    >,
+    data: Vec<SendableRecordBatchStream>,
+) -> Result<usize> {
+    let mut row_count = 0;
+    let mut join_set: JoinSet<Result<usize, DataFusionError>> = JoinSet::new();
+    for (mut data_stream, mut writer) in data.into_iter().zip(writers.into_iter()) {
+        join_set.spawn(async move {
+            let mut cnt = 0;
+            while let Some(batch) = data_stream.next().await.transpose()? {
+                cnt += batch.num_rows();
+                writer.write(&batch).await?;
+            }
+            writer.close().await?;
+            Ok(cnt)
+        });
+    }
+    while let Some(result) = join_set.join_next().await {
+        match result {
+            Ok(res) => {
+                row_count += res?;
+            } // propagate DataFusion error
+            Err(e) => {
+                if e.is_panic() {
+                    std::panic::resume_unwind(e.into_panic());
+                } else {
+                    unreachable!();
+                }
+            }
+        }
+    }
+
+    Ok(row_count)
+}
+
+/// A buffer with interior mutability shared by the SerializedFileWriter and
+/// ObjectStore writer
+#[derive(Clone)]
+struct SharedBuffer {
+    /// The inner buffer for reading and writing
+    ///
+    /// The lock is used to obtain internal mutability, so no worry about the
+    /// lock contention.
+    buffer: Arc<futures::lock::Mutex<Vec<u8>>>,
+}
+
+impl SharedBuffer {
+    pub fn new(capacity: usize) -> Self {
+        Self {
+            buffer: Arc::new(futures::lock::Mutex::new(Vec::with_capacity(capacity))),
+        }
+    }
+}
+
+impl Write for SharedBuffer {
+    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
+        let mut buffer = self.buffer.try_lock().unwrap();
+        Write::write(&mut *buffer, buf)
+    }
+
+    fn flush(&mut self) -> std::io::Result<()> {
+        let mut buffer = self.buffer.try_lock().unwrap();
+        Write::flush(&mut *buffer)
+    }
+}
+
 #[cfg(test)]
 pub(crate) mod test_util {
     use super::*;
@@ -993,7 +1101,7 @@ pub(crate) mod test_util {
         Ok((meta, files))
     }
 
-    //// write batches chunk_size rows at a time
+    /// write batches chunk_size rows at a time
     fn write_in_chunks<W: std::io::Write + Send>(
         writer: &mut ArrowWriter<W>,
         batch: &RecordBatch,
@@ -1018,7 +1126,6 @@ mod tests {
     use super::*;
 
     use crate::datasource::file_format::parquet::test_util::store_parquet;
-    use crate::datasource::physical_plan::get_scan_files;
     use crate::physical_plan::metrics::MetricValue;
     use crate::prelude::{SessionConfig, SessionContext};
     use arrow::array::{Array, ArrayRef, StringArray};
@@ -1271,7 +1378,7 @@ mod tests {
     #[tokio::test]
     async fn read_small_batches() -> Result<()> {
         let config = SessionConfig::new().with_batch_size(2);
-        let session_ctx = SessionContext::with_config(config);
+        let session_ctx = SessionContext::new_with_config(config);
         let state = session_ctx.state();
         let task_ctx = state.task_ctx();
         let projection = None;
@@ -1299,7 +1406,7 @@ mod tests {
     #[tokio::test]
     async fn capture_bytes_scanned_metric() -> Result<()> {
         let config = SessionConfig::new().with_batch_size(2);
-        let session = SessionContext::with_config(config);
+        let session = SessionContext::new_with_config(config);
         let ctx = session.state();
 
         // Read the full file
@@ -1613,25 +1720,6 @@ mod tests {
         Ok(())
     }
 
-    #[tokio::test]
-    async fn test_get_scan_files() -> Result<()> {
-        let session_ctx = SessionContext::new();
-        let state = session_ctx.state();
-        let projection = Some(vec![9]);
-        let exec = get_exec(&state, "alltypes_plain.parquet", projection, None).await?;
-        let scan_files = get_scan_files(exec)?;
-        assert_eq!(scan_files.len(), 1);
-        assert_eq!(scan_files[0].len(), 1);
-        assert_eq!(scan_files[0][0].len(), 1);
-        assert!(scan_files[0][0][0]
-            .object_meta
-            .location
-            .to_string()
-            .contains("alltypes_plain.parquet"));
-
-        Ok(())
-    }
-
     fn check_page_index_validation(
         page_index: Option<&ParquetColumnIndex>,
         offset_index: Option<&ParquetOffsetIndex>,
