@@ -22,7 +22,6 @@ use crate::datasource::{
     listing::{FileRange, PartitionedFile},
     object_store::ObjectStoreUrl,
 };
-use crate::physical_plan::ExecutionPlan;
 use crate::{
     error::{DataFusionError, Result},
     scalar::ScalarValue,
@@ -33,7 +32,7 @@ use arrow::buffer::Buffer;
 use arrow::datatypes::{ArrowNativeType, UInt16Type};
 use arrow_array::{ArrayRef, DictionaryArray, RecordBatch};
 use arrow_schema::{DataType, Field, Schema, SchemaRef};
-use datafusion_common::tree_node::{TreeNode, VisitRecursion};
+use datafusion_common::exec_err;
 use datafusion_common::{ColumnStatistics, Statistics};
 use datafusion_physical_expr::LexOrdering;
 
@@ -69,22 +68,6 @@ pub fn wrap_partition_value_in_dict(val: ScalarValue) -> ScalarValue {
     ScalarValue::Dictionary(Box::new(DataType::UInt16), Box::new(val))
 }
 
-/// Get all of the [`PartitionedFile`] to be scanned for an [`ExecutionPlan`]
-pub fn get_scan_files(
-    plan: Arc<dyn ExecutionPlan>,
-) -> Result<Vec<Vec<Vec<PartitionedFile>>>> {
-    let mut collector: Vec<Vec<Vec<PartitionedFile>>> = vec![];
-    plan.apply(&mut |plan| {
-        if let Some(file_scan_config) = plan.file_scan_config() {
-            collector.push(file_scan_config.file_groups.clone());
-            Ok(VisitRecursion::Skip)
-        } else {
-            Ok(VisitRecursion::Continue)
-        }
-    })?;
-    Ok(collector)
-}
-
 /// The base configurations to provide when creating a physical plan for
 /// any given file format.
 #[derive(Clone)]
@@ -331,11 +314,11 @@ impl PartitionColumnProjector {
             self.projected_schema.fields().len() - self.projected_partition_indexes.len();
 
         if file_batch.columns().len() != expected_cols {
-            return Err(DataFusionError::Execution(format!(
+            return exec_err!(
                 "Unexpected batch schema from file, expected {} cols but got {}",
                 expected_cols,
                 file_batch.columns().len()
-            )));
+            );
         }
         let mut cols = file_batch.columns().to_vec();
         for &(pidx, sidx) in &self.projected_partition_indexes {
@@ -344,7 +327,7 @@ impl PartitionColumnProjector {
             // check if user forgot to dict-encode the partition value
             let field = self.projected_schema.field(sidx);
             let expected_data_type = field.data_type();
-            let actual_data_type = partition_value.get_datatype();
+            let actual_data_type = partition_value.data_type();
             if let DataType::Dictionary(key_type, _) = expected_data_type {
                 if !matches!(actual_data_type, DataType::Dictionary(_, _)) {
                     warn!("Partition value for column {} was not dictionary-encoded, applied auto-fix.", field.name());
@@ -445,7 +428,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_i8,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::Int16 => {
@@ -453,7 +436,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_i16,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::Int32 => {
@@ -461,7 +444,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_i32,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::Int64 => {
@@ -469,7 +452,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_i64,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::UInt8 => {
@@ -477,7 +460,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_u8,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::UInt16 => {
@@ -485,7 +468,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_u16,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::UInt32 => {
@@ -493,7 +476,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_u32,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             DataType::UInt64 => {
@@ -501,7 +484,7 @@ fn create_output_array(
                     &mut key_buffer_cache.gen_u64,
                     dict_val,
                     len,
-                    val.get_datatype(),
+                    val.data_type(),
                 );
             }
             _ => {}
@@ -513,11 +496,10 @@ fn create_output_array(
 
 #[cfg(test)]
 mod tests {
+    use arrow_array::Int32Array;
+
     use super::*;
-    use crate::{
-        test::{build_table_i32, columns},
-        test_util::aggr_test_schema,
-    };
+    use crate::{test::columns, test_util::aggr_test_schema};
 
     #[test]
     fn physical_plan_config_no_projection() {
@@ -664,7 +646,7 @@ mod tests {
                 ],
             )
             .expect("Projection of partition columns into record batch failed");
-        let expected = vec![
+        let expected = [
             "+---+----+----+------+-----+",
             "| a | b  | c  | year | day |",
             "+---+----+----+------+-----+",
@@ -698,7 +680,7 @@ mod tests {
                 ],
             )
             .expect("Projection of partition columns into record batch failed");
-        let expected = vec![
+        let expected = [
             "+---+-----+----+------+-----+",
             "| a | b   | c  | year | day |",
             "+---+-----+----+------+-----+",
@@ -734,7 +716,7 @@ mod tests {
                 ],
             )
             .expect("Projection of partition columns into record batch failed");
-        let expected = vec![
+        let expected = [
             "+---+---+---+------+-----+",
             "| a | b | c | year | day |",
             "+---+---+---+------+-----+",
@@ -762,7 +744,7 @@ mod tests {
                 ],
             )
             .expect("Projection of partition columns into record batch failed");
-        let expected = vec![
+        let expected = [
             "+---+----+----+------+-----+",
             "| a | b  | c  | year | day |",
             "+---+----+----+------+-----+",
@@ -793,4 +775,27 @@ mod tests {
             infinite_source: false,
         }
     }
+
+    /// returns record batch with 3 columns of i32 in memory
+    pub fn build_table_i32(
+        a: (&str, &Vec<i32>),
+        b: (&str, &Vec<i32>),
+        c: (&str, &Vec<i32>),
+    ) -> RecordBatch {
+        let schema = Schema::new(vec![
+            Field::new(a.0, DataType::Int32, false),
+            Field::new(b.0, DataType::Int32, false),
+            Field::new(c.0, DataType::Int32, false),
+        ]);
+
+        RecordBatch::try_new(
+            Arc::new(schema),
+            vec![
+                Arc::new(Int32Array::from(a.1.clone())),
+                Arc::new(Int32Array::from(b.1.clone())),
+                Arc::new(Int32Array::from(c.1.clone())),
+            ],
+        )
+        .unwrap()
+    }
 }
