@@ -390,10 +390,6 @@ impl ExecutionPlan for ParquetExec {
     fn statistics(&self) -> Statistics {
         self.projected_statistics.clone()
     }
-
-    fn file_scan_config(&self) -> Option<&FileScanConfig> {
-        Some(&self.base_config)
-    }
 }
 
 /// Implements [`FileOpener`] for a parquet file
@@ -736,6 +732,7 @@ mod tests {
     // See also `parquet_exec` integration test
 
     use super::*;
+    use crate::dataframe::DataFrameWriteOptions;
     use crate::datasource::file_format::options::CsvReadOptions;
     use crate::datasource::file_format::parquet::test_util::store_parquet;
     use crate::datasource::file_format::test_util::scan_format;
@@ -932,10 +929,10 @@ mod tests {
         let df = ctx.read_csv("tests/data/corrupt.csv", options).await?;
         let out_dir_url = "file://local/out";
         let e = df
-            .write_parquet(out_dir_url, None)
+            .write_parquet(out_dir_url, DataFrameWriteOptions::new(), None)
             .await
             .expect_err("should fail because input file does not match inferred schema");
-        assert_eq!("Arrow error: Parser error: Error while parsing value d for column 0 at line 4", format!("{e}"));
+        assert_eq!(e.strip_backtrace(), "Arrow error: Parser error: Error while parsing value d for column 0 at line 4");
         Ok(())
     }
 
@@ -1002,7 +999,7 @@ mod tests {
             .round_trip_to_batches(vec![batch1, batch2])
             .await
             .unwrap();
-        let expected = vec![
+        let expected = [
             "+-----+----+----+",
             "| c1  | c2 | c3 |",
             "+-----+----+----+",
@@ -1037,7 +1034,7 @@ mod tests {
             .round_trip_to_batches(vec![batch1, batch2])
             .await
             .unwrap();
-        let expected = vec![
+        let expected = [
             "+-----+----+----+",
             "| c1  | c3 | c2 |",
             "+-----+----+----+",
@@ -1075,7 +1072,7 @@ mod tests {
             .round_trip_to_batches(vec![batch1, batch2])
             .await
             .unwrap();
-        let expected = vec![
+        let expected = [
             "+-----+----+----+",
             "| c1  | c3 | c2 |",
             "+-----+----+----+",
@@ -1114,7 +1111,7 @@ mod tests {
             .round_trip(vec![batch1, batch2])
             .await;
 
-        let expected = vec![
+        let expected = [
             "+----+----+----+",
             "| c1 | c3 | c2 |",
             "+----+----+----+",
@@ -1156,7 +1153,7 @@ mod tests {
             .round_trip_to_batches(vec![batch1, batch2])
             .await
             .unwrap();
-        let expected = vec![
+        let expected = [
             "+-----+-----+",
             "| c1  | c4  |",
             "+-----+-----+",
@@ -1230,7 +1227,7 @@ mod tests {
         // a null array, then the pruning predicate (currently) can not be applied.
         // In a real query where this predicate was pushed down from a filter stage instead of created directly in the `ParquetExec`,
         // the filter stage would be preserved as a separate execution plan stage so the actual query results would be as expected.
-        let expected = vec![
+        let expected = [
             "+-----+----+",
             "| c1  | c2 |",
             "+-----+----+",
@@ -1267,7 +1264,7 @@ mod tests {
             .round_trip(vec![batch1, batch2])
             .await;
 
-        let expected = vec![
+        let expected = [
             "+----+----+",
             "| c1 | c2 |",
             "+----+----+",
@@ -1374,7 +1371,7 @@ mod tests {
             .await
             .unwrap();
 
-        let expected = vec![
+        let expected = [
             "+-----+----+",
             "| c1  | c2 |",
             "+-----+----+",
@@ -1405,7 +1402,7 @@ mod tests {
             .await
             .unwrap();
 
-        let expected = vec![
+        let expected = [
             "+-----+----+",
             "| c1  | c2 |",
             "+-----+----+",
@@ -1651,7 +1648,7 @@ mod tests {
         let mut results = parquet_exec.execute(0, task_ctx)?;
         let batch = results.next().await.unwrap()?;
         assert_eq!(batch.schema().as_ref(), &expected_schema);
-        let expected = vec![
+        let expected = [
             "+----+----------+-------------+-------+-----+",
             "| id | bool_col | tinyint_col | month | day |",
             "+----+----------+-------------+-------+-----+",
@@ -1742,14 +1739,12 @@ mod tests {
 
         // assert the batches and some metrics
         #[rustfmt::skip]
-        let expected = vec![
-            "+-----+",
+        let expected = ["+-----+",
             "| int |",
             "+-----+",
             "| 4   |",
             "| 5   |",
-            "+-----+",
-        ];
+            "+-----+"];
         assert_batches_sorted_eq!(expected, &rt.batches.unwrap());
         assert_eq!(get_value(&metrics, "page_index_rows_filtered"), 4);
         assert!(
@@ -1786,7 +1781,7 @@ mod tests {
         let metrics = rt.parquet_exec.metrics().unwrap();
 
         // assert the batches and some metrics
-        let expected = vec![
+        let expected = [
             "+-----+", "| c1  |", "+-----+", "| Foo |", "| zzz |", "+-----+",
         ];
         assert_batches_sorted_eq!(expected, &rt.batches.unwrap());
@@ -1933,8 +1928,9 @@ mod tests {
         // create partitioned input file and context
         let tmp_dir = TempDir::new()?;
         // let mut ctx = create_ctx(&tmp_dir, 4).await?;
-        let ctx =
-            SessionContext::with_config(SessionConfig::new().with_target_partitions(8));
+        let ctx = SessionContext::new_with_config(
+            SessionConfig::new().with_target_partitions(8),
+        );
         let schema = populate_csv_partitions(&tmp_dir, 4, ".csv")?;
         // register csv file with the execution context
         ctx.register_csv(
@@ -1953,7 +1949,8 @@ mod tests {
         let out_dir = tmp_dir.as_ref().to_str().unwrap().to_string() + "/out";
         let out_dir_url = "file://local/out";
         let df = ctx.sql("SELECT c1, c2 FROM test").await?;
-        df.write_parquet(out_dir_url, None).await?;
+        df.write_parquet(out_dir_url, DataFrameWriteOptions::new(), None)
+            .await?;
         // write_parquet(&mut ctx, "SELECT c1, c2 FROM test", &out_dir, None).await?;
 
         // create a new context and verify that the results were saved to a partitioned parquet file
