@@ -50,74 +50,59 @@ impl PhysicalOptimizerRule for CombinePartialFinalAggregate {
         _config: &ConfigOptions,
     ) -> Result<Arc<dyn ExecutionPlan>> {
         plan.transform_down(&|plan| {
-            let transformed = plan.as_any().downcast_ref::<AggregateExec>().and_then(
-                |AggregateExec {
-                     mode: final_mode,
-                     input: final_input,
-                     group_by: final_group_by,
-                     aggr_expr: final_aggr_expr,
-                     filter_expr: final_filter_expr,
-                     ..
-                 }| {
-                    if matches!(
-                        final_mode,
-                        AggregateMode::Final | AggregateMode::FinalPartitioned
-                    ) {
-                        final_input
-                            .as_any()
-                            .downcast_ref::<AggregateExec>()
-                            .and_then(
-                                |AggregateExec {
-                                     mode: input_mode,
-                                     input: partial_input,
-                                     group_by: input_group_by,
-                                     aggr_expr: input_aggr_expr,
-                                     filter_expr: input_filter_expr,
-                                     order_by_expr: input_order_by_expr,
-                                     input_schema,
-                                     ..
-                                 }| {
-                                    if matches!(input_mode, AggregateMode::Partial)
-                                        && can_combine(
-                                            (
-                                                final_group_by,
-                                                final_aggr_expr,
-                                                final_filter_expr,
-                                            ),
-                                            (
-                                                input_group_by,
-                                                input_aggr_expr,
-                                                input_filter_expr,
-                                            ),
-                                        )
-                                    {
-                                        let mode = if *final_mode == AggregateMode::Final
-                                        {
-                                            AggregateMode::Single
-                                        } else {
-                                            AggregateMode::SinglePartitioned
-                                        };
+            let transformed =
+                plan.as_any()
+                    .downcast_ref::<AggregateExec>()
+                    .and_then(|agg_exec| {
+                        if matches!(
+                            agg_exec.mode(),
+                            AggregateMode::Final | AggregateMode::FinalPartitioned
+                        ) {
+                            agg_exec
+                                .input()
+                                .as_any()
+                                .downcast_ref::<AggregateExec>()
+                                .and_then(|input_agg_exec| {
+                                    if matches!(
+                                        input_agg_exec.mode(),
+                                        AggregateMode::Partial
+                                    ) && can_combine(
+                                        (
+                                            agg_exec.group_by(),
+                                            agg_exec.aggr_expr(),
+                                            agg_exec.filter_expr(),
+                                        ),
+                                        (
+                                            input_agg_exec.group_by(),
+                                            input_agg_exec.aggr_expr(),
+                                            input_agg_exec.filter_expr(),
+                                        ),
+                                    ) {
+                                        let mode =
+                                            if agg_exec.mode() == &AggregateMode::Final {
+                                                AggregateMode::Single
+                                            } else {
+                                                AggregateMode::SinglePartitioned
+                                            };
                                         AggregateExec::try_new(
                                             mode,
-                                            input_group_by.clone(),
-                                            input_aggr_expr.to_vec(),
-                                            input_filter_expr.to_vec(),
-                                            input_order_by_expr.to_vec(),
-                                            partial_input.clone(),
-                                            input_schema.clone(),
+                                            input_agg_exec.group_by().clone(),
+                                            input_agg_exec.aggr_expr().to_vec(),
+                                            input_agg_exec.filter_expr().to_vec(),
+                                            input_agg_exec.order_by_expr().to_vec(),
+                                            input_agg_exec.input().clone(),
+                                            input_agg_exec.input_schema().clone(),
                                         )
                                         .ok()
                                         .map(Arc::new)
                                     } else {
                                         None
                                     }
-                                },
-                            )
-                    } else {
-                        None
-                    }
-                },
-            );
+                                })
+                        } else {
+                            None
+                        }
+                    });
 
             Ok(if let Some(transformed) = transformed {
                 Transformed::Yes(transformed)
