@@ -1211,7 +1211,7 @@ mod tests {
 
         let batch =
             build_statistics_record_batch(&statistics, &required_columns).unwrap();
-        let expected = vec![
+        let expected = [
             "+--------+--------+--------+--------+",
             "| s1_min | s2_max | s3_max | s3_min |",
             "+--------+--------+--------+--------+",
@@ -1250,7 +1250,7 @@ mod tests {
 
         let batch =
             build_statistics_record_batch(&statistics, &required_columns).unwrap();
-        let expected = vec![
+        let expected = [
             "+-------------------------------+",
             "| s1_min                        |",
             "+-------------------------------+",
@@ -1296,7 +1296,7 @@ mod tests {
 
         let batch =
             build_statistics_record_batch(&statistics, &required_columns).unwrap();
-        let expected = vec![
+        let expected = [
             "+--------+",
             "| s1_min |",
             "+--------+",
