@@ -396,7 +396,7 @@ async fn run_window_test(
     let mut rng = StdRng::seed_from_u64(random_seed);
     let schema = input1[0].schema();
     let session_config = SessionConfig::new().with_batch_size(50);
-    let ctx = SessionContext::with_config(session_config);
+    let ctx = SessionContext::new_with_config(session_config);
     let (window_fn, args, fn_name) = get_random_function(&schema, &mut rng, is_linear);
 
     let window_frame = get_random_window_frame(&mut rng, is_linear);
@@ -550,7 +550,7 @@ fn make_staggered_batches<const STREAM: bool>(
     let input1 = Int32Array::from_iter_values(input123.iter().map(|k| k.0));
     let input2 = Int32Array::from_iter_values(input123.iter().map(|k| k.1));
     let input3 = Int32Array::from_iter_values(input123.iter().map(|k| k.2));
-    let input4 = Int32Array::from_iter_values(input4.into_iter());
+    let input4 = Int32Array::from_iter_values(input4);
 
     // split into several record batches
     let mut remainder = RecordBatch::try_from_iter(vec![
