@@ -28,8 +28,8 @@ use arrow::{
     datatypes::{DataType, Field},
 };
 use datafusion_common::{
-    downcast_value, internal_err, not_impl_err, plan_err, DataFusionError, Result,
-    ScalarValue,
+    downcast_value, exec_err, internal_err, not_impl_err, plan_err, DataFusionError,
+    Result, ScalarValue,
 };
 use datafusion_expr::Accumulator;
 use std::{any::Any, iter, sync::Arc};
@@ -147,7 +147,7 @@ fn validate_input_percentile_expr(expr: &Arc<dyn PhysicalExpr>) -> Result<f64> {
         ScalarValue::Float64(Some(q)) => *q,
         got => return not_impl_err!(
             "Percentile value for 'APPROX_PERCENTILE_CONT' must be Float32 or Float64 literal (got data type {})",
-            got.get_datatype()
+            got.data_type()
         )
     };
 
@@ -182,7 +182,7 @@ fn validate_input_max_size_expr(expr: &Arc<dyn PhysicalExpr>) -> Result<usize> {
         ScalarValue::Int8(Some(q)) if *q > 0 => *q as usize,
         got => return not_impl_err!(
             "Tdigest max_size value for 'APPROX_PERCENTILE_CONT' must be UInt > 0 literal (got data type {}).",
-            got.get_datatype()
+            got.data_type()
         )
     };
     Ok(max_size)
@@ -394,9 +394,7 @@ impl Accumulator for ApproxPercentileAccumulator {
 
     fn evaluate(&self) -> Result<ScalarValue> {
         if self.digest.count() == 0.0 {
-            return Err(DataFusionError::Execution(
-                "aggregate function needs at least one non-null element".to_string(),
-            ));
+            return exec_err!("aggregate function needs at least one non-null element");
         }
         let q = self.digest.estimate_quantile(self.percentile);
 
