@@ -18,17 +18,21 @@
 use crate::expressions::format_state_name;
 use arrow::datatypes::{DataType, Field};
 use std::any::Any;
-use std::fmt::Debug;
 use std::sync::Arc;
 
 use ahash::RandomState;
 use arrow::array::{Array, ArrayRef};
+use arrow_array::cast::AsArray;
+use arrow_array::types::*;
+use arrow_array::{ArrowNativeTypeOp, ArrowPrimitiveType};
+use arrow_buffer::{ArrowNativeType, ToByteSlice};
 use std::collections::HashSet;
 
+use crate::aggregate::sum::downcast_sum;
 use crate::aggregate::utils::down_cast_any_ref;
 use crate::{AggregateExpr, PhysicalExpr};
-use datafusion_common::ScalarValue;
-use datafusion_common::{internal_err, DataFusionError, Result};
+use datafusion_common::{not_impl_err, DataFusionError, Result, ScalarValue};
+use datafusion_expr::type_coercion::aggregates::sum_return_type;
 use datafusion_expr::Accumulator;
 
 /// Expression for a SUM(DISTINCT) aggregation.
@@ -49,6 +53,7 @@ impl DistinctSum {
         name: String,
         data_type: DataType,
     ) -> Self {
+        let data_type = sum_return_type(&data_type).unwrap();
         Self {
             name,
             data_type,
@@ -84,7 +89,12 @@ impl AggregateExpr for DistinctSum {
     }
 
     fn create_accumulator(&self) -> Result<Box<dyn Accumulator>> {
-        Ok(Box::new(DistinctSumAccumulator::try_new(&self.data_type)?))
+        macro_rules! helper {
+            ($t:ty, $dt:expr) => {
+                Ok(Box::new(DistinctSumAccumulator::<$t>::try_new(&$dt)?))
+            };
+        }
+        downcast_sum!(self, helper)
     }
 }
 
@@ -106,29 +116,56 @@ impl PartialEq<dyn Any> for DistinctSum {
     }
 }
 
-#[derive(Debug)]
-struct DistinctSumAccumulator {
-    hash_values: HashSet<ScalarValue, RandomState>,
+/// A wrapper around a type to provide hash for floats
+#[derive(Copy, Clone)]
+struct Hashable<T>(T);
+
+impl<T: ToByteSlice> std::hash::Hash for Hashable<T> {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.0.to_byte_slice().hash(state)
+    }
+}
+
+impl<T: ArrowNativeTypeOp> PartialEq for Hashable<T> {
+    fn eq(&self, other: &Self) -> bool {
+        self.0.is_eq(other.0)
+    }
+}
+
+impl<T: ArrowNativeTypeOp> Eq for Hashable<T> {}
+
+struct DistinctSumAccumulator<T: ArrowPrimitiveType> {
+    values: HashSet<Hashable<T::Native>, RandomState>,
     data_type: DataType,
 }
-impl DistinctSumAccumulator {
+
+impl<T: ArrowPrimitiveType> std::fmt::Debug for DistinctSumAccumulator<T> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "DistinctSumAccumulator({})", self.data_type)
+    }
+}
+
+impl<T: ArrowPrimitiveType> DistinctSumAccumulator<T> {
     pub fn try_new(data_type: &DataType) -> Result<Self> {
         Ok(Self {
-            hash_values: HashSet::default(),
+            values: HashSet::default(),
             data_type: data_type.clone(),
         })
     }
 }
 
-impl Accumulator for DistinctSumAccumulator {
+impl<T: ArrowPrimitiveType> Accumulator for DistinctSumAccumulator<T> {
     fn state(&self) -> Result<Vec<ScalarValue>> {
         // 1. Stores aggregate state in `ScalarValue::List`
         // 2. Constructs `ScalarValue::List` state from distinct numeric stored in hash set
         let state_out = {
             let mut distinct_values = Vec::new();
-            self.hash_values
-                .iter()
-                .for_each(|distinct_value| distinct_values.push(distinct_value.clone()));
+            self.values.iter().for_each(|distinct_value| {
+                distinct_values.push(ScalarValue::new_primitive::<T>(
+                    Some(distinct_value.0),
+                    &self.data_type,
+                ))
+            });
             vec![ScalarValue::new_list(
                 Some(distinct_values),
                 self.data_type.clone(),
@@ -142,62 +179,49 @@ impl Accumulator for DistinctSumAccumulator {
             return Ok(());
         }
 
-        let arr = &values[0];
-        (0..values[0].len()).try_for_each(|index| {
-            if !arr.is_null(index) {
-                let v = ScalarValue::try_from_array(arr, index)?;
-                self.hash_values.insert(v);
+        let array = values[0].as_primitive::<T>();
+        match array.nulls().filter(|x| x.null_count() > 0) {
+            Some(n) => {
+                for idx in n.valid_indices() {
+                    self.values.insert(Hashable(array.value(idx)));
+                }
             }
-            Ok(())
-        })
+            None => array.values().iter().for_each(|x| {
+                self.values.insert(Hashable(*x));
+            }),
+        }
+        Ok(())
     }
 
     fn merge_batch(&mut self, states: &[ArrayRef]) -> Result<()> {
-        if states.is_empty() {
-            return Ok(());
+        for x in states[0].as_list::<i32>().iter().flatten() {
+            self.update_batch(&[x])?
         }
-
-        let arr = &states[0];
-        (0..arr.len()).try_for_each(|index| {
-            let scalar = ScalarValue::try_from_array(arr, index)?;
-
-            if let ScalarValue::List(Some(scalar), _) = scalar {
-                scalar.iter().for_each(|scalar| {
-                    if !ScalarValue::is_null(scalar) {
-                        self.hash_values.insert(scalar.clone());
-                    }
-                });
-            } else {
-                return internal_err!("Unexpected accumulator state");
-            }
-            Ok(())
-        })
+        Ok(())
     }
 
     fn evaluate(&self) -> Result<ScalarValue> {
-        let mut sum_value = ScalarValue::try_from(&self.data_type)?;
-        for distinct_value in self.hash_values.iter() {
-            sum_value = sum_value.add(distinct_value)?;
+        let mut acc = T::Native::usize_as(0);
+        for distinct_value in self.values.iter() {
+            acc = acc.add_wrapping(distinct_value.0)
         }
-        Ok(sum_value)
+        let v = (!self.values.is_empty()).then_some(acc);
+        Ok(ScalarValue::new_primitive::<T>(v, &self.data_type))
     }
 
     fn size(&self) -> usize {
-        std::mem::size_of_val(self) + ScalarValue::size_of_hashset(&self.hash_values)
-            - std::mem::size_of_val(&self.hash_values)
-            + self.data_type.size()
-            - std::mem::size_of_val(&self.data_type)
+        std::mem::size_of_val(self)
+            + self.values.capacity() * std::mem::size_of::<T::Native>()
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::expressions::col;
-    use crate::expressions::tests::aggregate;
-    use arrow::record_batch::RecordBatch;
-    use arrow::{array::*, datatypes::*};
+    use crate::expressions::tests::assert_aggregate;
+    use arrow::array::*;
     use datafusion_common::Result;
+    use datafusion_expr::AggregateFunction;
 
     fn run_update_batch(
         return_type: DataType,
@@ -211,26 +235,6 @@ mod tests {
         Ok((accum.state()?, accum.evaluate()?))
     }
 
-    macro_rules! generic_test_sum_distinct {
-        ($ARRAY:expr, $DATATYPE:expr, $EXPECTED:expr) => {{
-            let schema = Schema::new(vec![Field::new("a", $DATATYPE, true)]);
-
-            let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![$ARRAY])?;
-
-            let agg = Arc::new(DistinctSum::new(
-                vec![col("a", &schema)?],
-                "count_distinct_a".to_string(),
-                $EXPECTED.get_datatype(),
-            ));
-            let actual = aggregate(&batch, agg)?;
-            let expected = ScalarValue::from($EXPECTED);
-
-            assert_eq!(expected, actual);
-
-            Ok(())
-        }};
-    }
-
     #[test]
     fn sum_distinct_update_batch() -> Result<()> {
         let array_int64: ArrayRef = Arc::new(Int64Array::from(vec![1, 1, 3]));
@@ -244,7 +248,7 @@ mod tests {
     }
 
     #[test]
-    fn sum_distinct_i32_with_nulls() -> Result<()> {
+    fn sum_distinct_i32_with_nulls() {
         let array = Arc::new(Int32Array::from(vec![
             Some(1),
             Some(1),
@@ -253,11 +257,11 @@ mod tests {
             Some(2),
             Some(3),
         ]));
-        generic_test_sum_distinct!(array, DataType::Int32, ScalarValue::from(6_i32))
+        assert_aggregate(array, AggregateFunction::Sum, true, 6_i64.into());
     }
 
     #[test]
-    fn sum_distinct_u32_with_nulls() -> Result<()> {
+    fn sum_distinct_u32_with_nulls() {
         let array: ArrayRef = Arc::new(UInt32Array::from(vec![
             Some(1_u32),
             Some(1_u32),
@@ -265,28 +269,30 @@ mod tests {
             Some(3_u32),
             None,
         ]));
-        generic_test_sum_distinct!(array, DataType::UInt32, ScalarValue::from(4_u32))
+        assert_aggregate(array, AggregateFunction::Sum, true, 4_u64.into());
     }
 
     #[test]
-    fn sum_distinct_f64() -> Result<()> {
+    fn sum_distinct_f64() {
         let array: ArrayRef =
             Arc::new(Float64Array::from(vec![1_f64, 1_f64, 3_f64, 3_f64, 3_f64]));
-        generic_test_sum_distinct!(array, DataType::Float64, ScalarValue::from(4_f64))
+        assert_aggregate(array, AggregateFunction::Sum, true, 4_f64.into());
     }
 
     #[test]
-    fn sum_distinct_decimal_with_nulls() -> Result<()> {
+    fn sum_distinct_decimal_with_nulls() {
         let array: ArrayRef = Arc::new(
             (1..6)
                 .map(|i| if i == 2 { None } else { Some(i % 2) })
                 .collect::<Decimal128Array>()
-                .with_precision_and_scale(35, 0)?,
+                .with_precision_and_scale(35, 0)
+                .unwrap(),
         );
-        generic_test_sum_distinct!(
+        assert_aggregate(
             array,
-            DataType::Decimal128(35, 0),
-            ScalarValue::Decimal128(Some(1), 38, 0)
-        )
+            AggregateFunction::Sum,
+            true,
+            ScalarValue::Decimal128(Some(1), 38, 0),
+        );
     }
 }
