@@ -91,13 +91,11 @@ impl ExprBoundaries {
         Self {
             column: Column::new(&col, index),
             interval: Interval::new(
-                IntervalBound::new(
+                IntervalBound::new_closed(
                     stats.min_value.clone().unwrap_or(ScalarValue::Null),
-                    false,
                 ),
-                IntervalBound::new(
+                IntervalBound::new_closed(
                     stats.max_value.clone().unwrap_or(ScalarValue::Null),
-                    false,
                 ),
             ),
             distinct_count: stats.distinct_count,
@@ -191,12 +189,15 @@ fn shrink_boundaries(
     })?;
     let final_result = graph.get_interval(*root_index);
 
+    // If during selectivity calculation we encounter an error, use 1.0 as cardinality estimate
+    // safest estimate(e.q largest possible value).
     let selectivity = calculate_selectivity(
         &final_result.lower.value,
         &final_result.upper.value,
         &target_boundaries,
         &initial_boundaries,
-    )?;
+    )
+    .unwrap_or(1.0);
 
     if !(0.0..=1.0).contains(&selectivity) {
         return internal_err!("Selectivity is out of limit: {}", selectivity);
