@@ -22,7 +22,9 @@ use std::sync::Arc;
 
 use crate::intervals::Interval;
 use crate::physical_expr::down_cast_any_ref;
+use crate::sort_properties::SortProperties;
 use crate::PhysicalExpr;
+
 use arrow::compute;
 use arrow::compute::{kernels, CastOptions};
 use arrow::datatypes::{DataType, Schema};
@@ -138,6 +140,11 @@ impl PhysicalExpr for CastExpr {
         // Add `self.cast_options` when hash is available
         // https://github.com/apache/arrow-rs/pull/4395
     }
+
+    /// A [`CastExpr`] preserves the ordering of its child.
+    fn get_ordering(&self, children: &[SortProperties]) -> SortProperties {
+        children[0]
+    }
 }
 
 impl PartialEq<dyn Any> for CastExpr {
@@ -346,13 +353,13 @@ mod tests {
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(20, 6),
-            vec![
+            [
                 Some(1_234_000),
                 Some(2_222_000),
                 Some(3_000),
                 Some(4_000_000),
                 Some(5_000_000),
-                None,
+                None
             ],
             None
         );
@@ -367,7 +374,7 @@ mod tests {
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![Some(123), Some(222), Some(0), Some(400), Some(500), None,],
+            [Some(123), Some(222), Some(0), Some(400), Some(500), None],
             None
         );
 
@@ -388,13 +395,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int8Array,
             DataType::Int8,
-            vec![
+            [
                 Some(1_i8),
                 Some(2_i8),
                 Some(3_i8),
                 Some(4_i8),
                 Some(5_i8),
-                None,
+                None
             ],
             None
         );
@@ -410,13 +417,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int16Array,
             DataType::Int16,
-            vec![
+            [
                 Some(1_i16),
                 Some(2_i16),
                 Some(3_i16),
                 Some(4_i16),
                 Some(5_i16),
-                None,
+                None
             ],
             None
         );
@@ -432,13 +439,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int32Array,
             DataType::Int32,
-            vec![
+            [
                 Some(1_i32),
                 Some(2_i32),
                 Some(3_i32),
                 Some(4_i32),
                 Some(5_i32),
-                None,
+                None
             ],
             None
         );
@@ -453,13 +460,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int64Array,
             DataType::Int64,
-            vec![
+            [
                 Some(1_i64),
                 Some(2_i64),
                 Some(3_i64),
                 Some(4_i64),
                 Some(5_i64),
-                None,
+                None
             ],
             None
         );
@@ -483,13 +490,13 @@ mod tests {
             DataType::Decimal128(10, 3),
             Float32Array,
             DataType::Float32,
-            vec![
+            [
                 Some(1.234_f32),
                 Some(2.222_f32),
                 Some(0.003_f32),
                 Some(4.0_f32),
                 Some(5.0_f32),
-                None,
+                None
             ],
             None
         );
@@ -504,13 +511,13 @@ mod tests {
             DataType::Decimal128(20, 6),
             Float64Array,
             DataType::Float64,
-            vec![
+            [
                 Some(0.001234_f64),
                 Some(0.002222_f64),
                 Some(0.000003_f64),
                 Some(0.004_f64),
                 Some(0.005_f64),
-                None,
+                None
             ],
             None
         );
@@ -526,7 +533,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(3, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),],
+            [Some(1), Some(2), Some(3), Some(4), Some(5)],
             None
         );
 
@@ -537,7 +544,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(5, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),],
+            [Some(1), Some(2), Some(3), Some(4), Some(5)],
             None
         );
 
@@ -548,7 +555,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(10, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),],
+            [Some(1), Some(2), Some(3), Some(4), Some(5)],
             None
         );
 
@@ -559,7 +566,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),],
+            [Some(1), Some(2), Some(3), Some(4), Some(5)],
             None
         );
 
@@ -570,7 +577,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 2),
-            vec![Some(100), Some(200), Some(300), Some(400), Some(500),],
+            [Some(100), Some(200), Some(300), Some(400), Some(500)],
             None
         );
 
@@ -581,7 +588,7 @@ mod tests {
             vec![1.5, 2.5, 3.0, 1.123_456_8, 5.50],
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![Some(150), Some(250), Some(300), Some(112), Some(550),],
+            [Some(150), Some(250), Some(300), Some(112), Some(550)],
             None
         );
 
@@ -592,12 +599,12 @@ mod tests {
             vec![1.5, 2.5, 3.0, 1.123_456_8, 5.50],
             Decimal128Array,
             DataType::Decimal128(20, 4),
-            vec![
+            [
                 Some(15000),
                 Some(25000),
                 Some(30000),
                 Some(11235),
-                Some(55000),
+                Some(55000)
             ],
             None
         );
@@ -612,7 +619,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             UInt32Array,
             DataType::UInt32,
-            vec![
+            [
                 Some(1_u32),
                 Some(2_u32),
                 Some(3_u32),
@@ -632,7 +639,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             StringArray,
             DataType::Utf8,
-            vec![Some("1"), Some("2"), Some("3"), Some("4"), Some("5")],
+            [Some("1"), Some("2"), Some("3"), Some("4"), Some("5")],
             None
         );
         Ok(())
