@@ -18,21 +18,25 @@
 //! Implementation of `InList` expressions: [`InListExpr`]
 
 use ahash::RandomState;
+use datafusion_common::exec_err;
 use std::any::Any;
 use std::fmt::Debug;
 use std::hash::{Hash, Hasher};
 use std::sync::Arc;
 
-use crate::hash_utils::HashValue;
 use crate::physical_expr::down_cast_any_ref;
 use crate::utils::expr_list_eq_any_order;
 use crate::PhysicalExpr;
 use arrow::array::*;
+use arrow::buffer::BooleanBuffer;
+use arrow::compute::kernels::boolean::{not, or_kleene};
+use arrow::compute::kernels::cmp::eq;
 use arrow::compute::take;
 use arrow::datatypes::*;
 use arrow::record_batch::RecordBatch;
 use arrow::util::bit_iterator::BitIndexIterator;
 use arrow::{downcast_dictionary_array, downcast_primitive_array};
+use datafusion_common::hash_utils::HashValue;
 use datafusion_common::{
     cast::{as_boolean_array, as_generic_binary_array, as_string_array},
     internal_err, not_impl_err, DataFusionError, Result, ScalarValue,
@@ -62,6 +66,7 @@ impl Debug for InListExpr {
 /// A type-erased container of array elements
 pub trait Set: Send + Sync {
     fn contains(&self, v: &dyn Array, negated: bool) -> Result<BooleanArray>;
+    fn has_nulls(&self) -> bool;
 }
 
 struct ArrayHashSet {
@@ -94,7 +99,7 @@ impl<T> Set for ArraySet<T>
 where
     T: Array + 'static,
     for<'a> &'a T: ArrayAccessor,
-    for<'a> <&'a T as ArrayAccessor>::Item: PartialEq + HashValue,
+    for<'a> <&'a T as ArrayAccessor>::Item: IsEqual,
 {
     fn contains(&self, v: &dyn Array, negated: bool) -> Result<BooleanArray> {
         downcast_dictionary_array! {
@@ -118,7 +123,7 @@ where
                         .hash_set
                         .map
                         .raw_entry()
-                        .from_hash(hash, |idx| in_array.value(*idx) == v)
+                        .from_hash(hash, |idx| in_array.value(*idx).is_equal(&v))
                         .is_some();
 
                     match contains {
@@ -130,6 +135,10 @@ where
             })
             .collect())
     }
+
+    fn has_nulls(&self) -> bool {
+        self.array.null_count() != 0
+    }
 }
 
 /// Computes an [`ArrayHashSet`] for the provided [`Array`] if there
@@ -141,7 +150,7 @@ where
 fn make_hash_set<T>(array: T) -> ArrayHashSet
 where
     T: ArrayAccessor,
-    T::Item: PartialEq + HashValue,
+    T::Item: IsEqual,
 {
     let state = RandomState::new();
     let mut map: HashMap<usize, (), ()> =
@@ -152,7 +161,7 @@ where
         let hash = value.hash_one(&state);
         if let RawEntryMut::Vacant(v) = map
             .raw_entry_mut()
-            .from_hash(hash, |x| array.value(*x) == value)
+            .from_hash(hash, |x| array.value(*x).is_equal(&value))
         {
             v.insert_with_hasher(hash, idx, (), |x| array.value(*x).hash_one(&state));
         }
@@ -207,9 +216,9 @@ fn evaluate_list(
         .iter()
         .map(|expr| {
             expr.evaluate(batch).and_then(|r| match r {
-                ColumnarValue::Array(_) => Err(DataFusionError::Execution(
-                    "InList expression must evaluate to a scalar".to_string(),
-                )),
+                ColumnarValue::Array(_) => {
+                    exec_err!("InList expression must evaluate to a scalar")
+                }
                 // Flatten dictionary values
                 ColumnarValue::Scalar(ScalarValue::Dictionary(_, v)) => Ok(*v),
                 ColumnarValue::Scalar(s) => Ok(s),
@@ -228,6 +237,40 @@ fn try_cast_static_filter_to_set(
     make_set(evaluate_list(list, &batch)?.as_ref())
 }
 
+/// Custom equality check function which is used with [`ArrayHashSet`] for existence check.
+trait IsEqual: HashValue {
+    fn is_equal(&self, other: &Self) -> bool;
+}
+
+impl<'a, T: IsEqual + ?Sized> IsEqual for &'a T {
+    fn is_equal(&self, other: &Self) -> bool {
+        T::is_equal(self, other)
+    }
+}
+
+macro_rules! is_equal {
+    ($($t:ty),+) => {
+        $(impl IsEqual for $t {
+            fn is_equal(&self, other: &Self) -> bool {
+                self == other
+            }
+        })*
+    };
+}
+is_equal!(i8, i16, i32, i64, i128, i256, u8, u16, u32, u64);
+is_equal!(bool, str, [u8]);
+
+macro_rules! is_equal_float {
+    ($($t:ty),+) => {
+        $(impl IsEqual for $t {
+            fn is_equal(&self, other: &Self) -> bool {
+                self.to_bits() == other.to_bits()
+            }
+        })*
+    };
+}
+is_equal_float!(half::f16, f32, f64);
+
 impl InListExpr {
     /// Create a new InList expression
     pub fn new(
@@ -287,16 +330,43 @@ impl PhysicalExpr for InListExpr {
     }
 
     fn nullable(&self, input_schema: &Schema) -> Result<bool> {
-        self.expr.nullable(input_schema)
+        if self.expr.nullable(input_schema)? {
+            return Ok(true);
+        }
+
+        if let Some(static_filter) = &self.static_filter {
+            Ok(static_filter.has_nulls())
+        } else {
+            for expr in &self.list {
+                if expr.nullable(input_schema)? {
+                    return Ok(true);
+                }
+            }
+            Ok(false)
+        }
     }
 
     fn evaluate(&self, batch: &RecordBatch) -> Result<ColumnarValue> {
-        let value = self.expr.evaluate(batch)?.into_array(1);
+        let value = self.expr.evaluate(batch)?;
         let r = match &self.static_filter {
-            Some(f) => f.contains(value.as_ref(), self.negated)?,
+            Some(f) => f.contains(value.into_array(1).as_ref(), self.negated)?,
             None => {
-                let list = evaluate_list(&self.list, batch)?;
-                make_set(list.as_ref())?.contains(value.as_ref(), self.negated)?
+                let value = value.into_array(batch.num_rows());
+                let found = self.list.iter().map(|expr| expr.evaluate(batch)).try_fold(
+                    BooleanArray::new(BooleanBuffer::new_unset(batch.num_rows()), None),
+                    |result, expr| -> Result<BooleanArray> {
+                        Ok(or_kleene(
+                            &result,
+                            &eq(&value, &expr?.into_array(batch.num_rows()))?,
+                        )?)
+                    },
+                )?;
+
+                if self.negated {
+                    not(&found)?
+                } else {
+                    found
+                }
             }
         };
         Ok(ColumnarValue::Array(Arc::new(r)))
@@ -419,9 +489,8 @@ mod tests {
     fn get_coerce_type(expr_type: &DataType, list_type: &[DataType]) -> Option<DataType> {
         list_type
             .iter()
-            .fold(Some(expr_type.clone()), |left, right_type| match left {
-                None => None,
-                Some(left_type) => comparison_coercion(&left_type, right_type),
+            .try_fold(expr_type.clone(), |left_type, right_type| {
+                comparison_coercion(&left_type, right_type)
             })
     }
 
@@ -609,50 +678,100 @@ mod tests {
     #[test]
     fn in_list_float64() -> Result<()> {
         let schema = Schema::new(vec![Field::new("a", DataType::Float64, true)]);
-        let a = Float64Array::from(vec![Some(0.0), Some(0.2), None]);
+        let a = Float64Array::from(vec![
+            Some(0.0),
+            Some(0.2),
+            None,
+            Some(f64::NAN),
+            Some(-f64::NAN),
+        ]);
         let col_a = col("a", &schema)?;
         let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![Arc::new(a)])?;
 
-        // expression: "a in (0.0, 0.2)"
+        // expression: "a in (0.0, 0.1)"
         let list = vec![lit(0.0f64), lit(0.1f64)];
         in_list!(
             batch,
             list,
             &false,
-            vec![Some(true), Some(false), None],
+            vec![Some(true), Some(false), None, Some(false), Some(false)],
             col_a.clone(),
             &schema
         );
 
-        // expression: "a not in (0.0, 0.2)"
+        // expression: "a not in (0.0, 0.1)"
         let list = vec![lit(0.0f64), lit(0.1f64)];
         in_list!(
             batch,
             list,
             &true,
-            vec![Some(false), Some(true), None],
+            vec![Some(false), Some(true), None, Some(true), Some(true)],
             col_a.clone(),
             &schema
         );
 
-        // expression: "a in (0.0, 0.2, NULL)"
+        // expression: "a in (0.0, 0.1, NULL)"
         let list = vec![lit(0.0f64), lit(0.1f64), lit(ScalarValue::Null)];
         in_list!(
             batch,
             list,
             &false,
-            vec![Some(true), None, None],
+            vec![Some(true), None, None, None, None],
             col_a.clone(),
             &schema
         );
 
-        // expression: "a not in (0.0, 0.2, NULL)"
+        // expression: "a not in (0.0, 0.1, NULL)"
         let list = vec![lit(0.0f64), lit(0.1f64), lit(ScalarValue::Null)];
         in_list!(
             batch,
             list,
             &true,
-            vec![Some(false), None, None],
+            vec![Some(false), None, None, None, None],
+            col_a.clone(),
+            &schema
+        );
+
+        // expression: "a in (0.0, 0.1, NaN)"
+        let list = vec![lit(0.0f64), lit(0.1f64), lit(f64::NAN)];
+        in_list!(
+            batch,
+            list,
+            &false,
+            vec![Some(true), Some(false), None, Some(true), Some(false)],
+            col_a.clone(),
+            &schema
+        );
+
+        // expression: "a not in (0.0, 0.1, NaN)"
+        let list = vec![lit(0.0f64), lit(0.1f64), lit(f64::NAN)];
+        in_list!(
+            batch,
+            list,
+            &true,
+            vec![Some(false), Some(true), None, Some(false), Some(true)],
+            col_a.clone(),
+            &schema
+        );
+
+        // expression: "a in (0.0, 0.1, -NaN)"
+        let list = vec![lit(0.0f64), lit(0.1f64), lit(-f64::NAN)];
+        in_list!(
+            batch,
+            list,
+            &false,
+            vec![Some(true), Some(false), None, Some(false), Some(true)],
+            col_a.clone(),
+            &schema
+        );
+
+        // expression: "a not in (0.0, 0.1, -NaN)"
+        let list = vec![lit(0.0f64), lit(0.1f64), lit(-f64::NAN)];
+        in_list!(
+            batch,
+            list,
+            &true,
+            vec![Some(false), Some(true), None, Some(true), Some(false)],
             col_a.clone(),
             &schema
         );
@@ -1044,4 +1163,103 @@ mod tests {
         );
         Ok(())
     }
+
+    #[test]
+    fn in_expr_with_multiple_element_in_list() -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Float64, true),
+            Field::new("b", DataType::Float64, true),
+            Field::new("c", DataType::Float64, true),
+        ]);
+        let a = Float64Array::from(vec![
+            Some(0.0),
+            Some(1.0),
+            Some(2.0),
+            Some(f64::NAN),
+            Some(-f64::NAN),
+        ]);
+        let b = Float64Array::from(vec![
+            Some(8.0),
+            Some(1.0),
+            Some(5.0),
+            Some(f64::NAN),
+            Some(3.0),
+        ]);
+        let c = Float64Array::from(vec![
+            Some(6.0),
+            Some(7.0),
+            None,
+            Some(5.0),
+            Some(-f64::NAN),
+        ]);
+        let col_a = col("a", &schema)?;
+        let col_b = col("b", &schema)?;
+        let col_c = col("c", &schema)?;
+        let batch = RecordBatch::try_new(
+            Arc::new(schema.clone()),
+            vec![Arc::new(a), Arc::new(b), Arc::new(c)],
+        )?;
+
+        let list = vec![col_b.clone(), col_c.clone()];
+        in_list!(
+            batch,
+            list.clone(),
+            &false,
+            vec![Some(false), Some(true), None, Some(true), Some(true)],
+            col_a.clone(),
+            &schema
+        );
+
+        in_list!(
+            batch,
+            list,
+            &true,
+            vec![Some(true), Some(false), None, Some(false), Some(false)],
+            col_a.clone(),
+            &schema
+        );
+
+        Ok(())
+    }
+
+    macro_rules! test_nullable {
+        ($COL:expr, $LIST:expr, $SCHEMA:expr, $EXPECTED:expr) => {{
+            let (cast_expr, cast_list_exprs) = in_list_cast($COL, $LIST, $SCHEMA)?;
+            let expr = in_list(cast_expr, cast_list_exprs, &false, $SCHEMA).unwrap();
+            let result = expr.nullable($SCHEMA)?;
+            assert_eq!($EXPECTED, result);
+        }};
+    }
+
+    #[test]
+    fn in_list_nullable() -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("c1_nullable", DataType::Int64, true),
+            Field::new("c2_non_nullable", DataType::Int64, false),
+        ]);
+
+        let c1_nullable = col("c1_nullable", &schema)?;
+        let c2_non_nullable = col("c2_non_nullable", &schema)?;
+
+        // static_filter has no nulls
+        let list = vec![lit(1_i64), lit(2_i64)];
+        test_nullable!(c1_nullable.clone(), list.clone(), &schema, true);
+        test_nullable!(c2_non_nullable.clone(), list.clone(), &schema, false);
+
+        // static_filter has nulls
+        let list = vec![lit(1_i64), lit(2_i64), lit(ScalarValue::Null)];
+        test_nullable!(c1_nullable.clone(), list.clone(), &schema, true);
+        test_nullable!(c2_non_nullable.clone(), list.clone(), &schema, true);
+
+        let list = vec![c1_nullable.clone()];
+        test_nullable!(c2_non_nullable.clone(), list.clone(), &schema, true);
+
+        let list = vec![c2_non_nullable.clone()];
+        test_nullable!(c1_nullable.clone(), list.clone(), &schema, true);
+
+        let list = vec![c2_non_nullable.clone(), c2_non_nullable.clone()];
+        test_nullable!(c2_non_nullable.clone(), list.clone(), &schema, false);
+
+        Ok(())
+    }
 }
