@@ -22,6 +22,7 @@ use std::hash::{Hash, Hasher};
 use std::sync::Arc;
 
 use crate::physical_expr::down_cast_any_ref;
+use crate::sort_properties::SortProperties;
 use crate::PhysicalExpr;
 
 use arrow::{
@@ -62,7 +63,7 @@ impl PhysicalExpr for Literal {
     }
 
     fn data_type(&self, _input_schema: &Schema) -> Result<DataType> {
-        Ok(self.value.get_datatype())
+        Ok(self.value.data_type())
     }
 
     fn nullable(&self, _input_schema: &Schema) -> Result<bool> {
@@ -88,6 +89,10 @@ impl PhysicalExpr for Literal {
         let mut s = state;
         self.hash(&mut s);
     }
+
+    fn get_ordering(&self, _children: &[SortProperties]) -> SortProperties {
+        SortProperties::Singleton
+    }
 }
 
 impl PartialEq<dyn Any> for Literal {
