@@ -22,6 +22,7 @@ mod binary;
 mod case;
 mod cast;
 mod column;
+mod datum;
 mod get_indexed_field;
 mod in_list;
 mod is_not_null;
@@ -59,7 +60,7 @@ pub use crate::aggregate::grouping::Grouping;
 pub use crate::aggregate::median::Median;
 pub use crate::aggregate::min_max::{Max, Min};
 pub use crate::aggregate::min_max::{MaxAccumulator, MinAccumulator};
-pub use crate::aggregate::regr::Regr;
+pub use crate::aggregate::regr::{Regr, RegrType};
 pub use crate::aggregate::stats::StatsType;
 pub use crate::aggregate::stddev::{Stddev, StddevPop};
 pub use crate::aggregate::sum::Sum;
@@ -101,7 +102,7 @@ pub use crate::PhysicalSortExpr;
 #[cfg(test)]
 pub(crate) mod tests {
     use crate::expressions::{col, create_aggregate_expr, try_cast};
-    use crate::AggregateExpr;
+    use crate::{AggregateExpr, EmitTo};
     use arrow::record_batch::RecordBatch;
     use arrow_array::ArrayRef;
     use arrow_schema::{Field, Schema};
@@ -111,11 +112,12 @@ pub(crate) mod tests {
     use datafusion_expr::AggregateFunction;
     use std::sync::Arc;
 
-    /// macro to perform an aggregation and verify the result.
+    /// macro to perform an aggregation using [`datafusion_expr::Accumulator`] and verify the
+    /// result.
     #[macro_export]
     macro_rules! generic_test_op {
         ($ARRAY:expr, $DATATYPE:expr, $OP:ident, $EXPECTED:expr) => {
-            generic_test_op!($ARRAY, $DATATYPE, $OP, $EXPECTED, $EXPECTED.get_datatype())
+            generic_test_op!($ARRAY, $DATATYPE, $OP, $EXPECTED, $EXPECTED.data_type())
         };
         ($ARRAY:expr, $DATATYPE:expr, $OP:ident, $EXPECTED:expr, $EXPECTED_DATATYPE:expr) => {{
             let schema = Schema::new(vec![Field::new("a", $DATATYPE, true)]);
@@ -136,10 +138,44 @@ pub(crate) mod tests {
         }};
     }
 
+    /// macro to perform an aggregation using [`crate::GroupsAccumulator`] and verify the result.
+    ///
+    /// The difference between this and the above `generic_test_op` is that the former checks
+    /// the old slow-path [`datafusion_expr::Accumulator`] implementation, while this checks
+    /// the new [`crate::GroupsAccumulator`] implementation.
+    #[macro_export]
+    macro_rules! generic_test_op_new {
+        ($ARRAY:expr, $DATATYPE:expr, $OP:ident, $EXPECTED:expr) => {
+            generic_test_op_new!(
+                $ARRAY,
+                $DATATYPE,
+                $OP,
+                $EXPECTED,
+                $EXPECTED.data_type().clone()
+            )
+        };
+        ($ARRAY:expr, $DATATYPE:expr, $OP:ident, $EXPECTED:expr, $EXPECTED_DATATYPE:expr) => {{
+            let schema = Schema::new(vec![Field::new("a", $DATATYPE, true)]);
+
+            let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![$ARRAY])?;
+
+            let agg = Arc::new(<$OP>::new(
+                col("a", &schema)?,
+                "bla".to_string(),
+                $EXPECTED_DATATYPE,
+            ));
+            let actual = aggregate_new(&batch, agg)?;
+            assert_eq!($EXPECTED, &actual);
+
+            Ok(()) as Result<(), DataFusionError>
+        }};
+    }
+
     /// Assert `function(array) == expected` performing any necessary type coercion
     pub fn assert_aggregate(
         array: ArrayRef,
         function: AggregateFunction,
+        distinct: bool,
         expected: ScalarValue,
     ) {
         let data_type = array.data_type();
@@ -159,7 +195,7 @@ pub(crate) mod tests {
 
         let schema = Schema::new(vec![Field::new("a", coerced[0].clone(), true)]);
         let agg =
-            create_aggregate_expr(&function, false, &[input], &[], &schema, "aggregate")
+            create_aggregate_expr(&function, distinct, &[input], &[], &schema, "agg")
                 .unwrap();
 
         let result = aggregate(&batch, agg).unwrap();
@@ -177,7 +213,7 @@ pub(crate) mod tests {
                 $DATATYPE2,
                 $OP,
                 $EXPECTED,
-                $EXPECTED.get_datatype()
+                $EXPECTED.data_type()
             )
         };
         ($ARRAY1:expr, $ARRAY2:expr, $DATATYPE1:expr, $DATATYPE2:expr, $OP:ident, $EXPECTED:expr, $EXPECTED_DATATYPE:expr) => {{
@@ -217,4 +253,20 @@ pub(crate) mod tests {
         accum.update_batch(&values)?;
         accum.evaluate()
     }
+
+    pub fn aggregate_new(
+        batch: &RecordBatch,
+        agg: Arc<dyn AggregateExpr>,
+    ) -> Result<ArrayRef> {
+        let mut accum = agg.create_groups_accumulator()?;
+        let expr = agg.expressions();
+        let values = expr
+            .iter()
+            .map(|e| e.evaluate(batch))
+            .map(|r| r.map(|v| v.into_array(batch.num_rows())))
+            .collect::<Result<Vec<_>>>()?;
+        let indices = vec![0; batch.num_rows()];
+        accum.update_batch(&values, &indices, None, 1)?;
+        accum.evaluate(EmitTo::All)
+    }
 }
