@@ -16,15 +16,11 @@
 // under the License.
 
 use arrow::array::Array;
-use arrow::compute::eq_dyn;
+use arrow::compute::kernels::cmp::eq;
 use arrow::compute::kernels::nullif::nullif;
-use datafusion_common::{
-    cast::as_boolean_array, internal_err, DataFusionError, Result, ScalarValue,
-};
+use datafusion_common::{internal_err, DataFusionError, Result, ScalarValue};
 use datafusion_expr::ColumnarValue;
 
-use super::binary::array_eq_scalar;
-
 /// Implements NULLIF(expr1, expr2)
 /// Args: 0 - left expr is any array
 ///       1 - if the left is equal to this expr2, then the result is NULL, otherwise left value is passed.
@@ -41,31 +37,23 @@ pub fn nullif_func(args: &[ColumnarValue]) -> Result<ColumnarValue> {
 
     match (lhs, rhs) {
         (ColumnarValue::Array(lhs), ColumnarValue::Scalar(rhs)) => {
-            let cond_array = array_eq_scalar(lhs, rhs)?;
-
-            let array = nullif(lhs, as_boolean_array(&cond_array)?)?;
+            let rhs = rhs.to_scalar();
+            let array = nullif(lhs, &eq(&lhs, &rhs)?)?;
 
             Ok(ColumnarValue::Array(array))
         }
         (ColumnarValue::Array(lhs), ColumnarValue::Array(rhs)) => {
-            // Get args0 == args1 evaluated and produce a boolean array
-            let cond_array = eq_dyn(lhs, rhs)?;
-
-            // Now, invoke nullif on the result
-            let array = nullif(lhs, as_boolean_array(&cond_array)?)?;
+            let array = nullif(lhs, &eq(&lhs, &rhs)?)?;
             Ok(ColumnarValue::Array(array))
         }
         (ColumnarValue::Scalar(lhs), ColumnarValue::Array(rhs)) => {
-            // Similar to Array-Array case, except of ScalarValue -> Array cast
             let lhs = lhs.to_array_of_size(rhs.len());
-            let cond_array = eq_dyn(&lhs, rhs)?;
-
-            let array = nullif(&lhs, as_boolean_array(&cond_array)?)?;
+            let array = nullif(&lhs, &eq(&lhs, &rhs)?)?;
             Ok(ColumnarValue::Array(array))
         }
         (ColumnarValue::Scalar(lhs), ColumnarValue::Scalar(rhs)) => {
             let val: ScalarValue = match lhs.eq(rhs) {
-                true => lhs.get_datatype().try_into()?,
+                true => lhs.data_type().try_into()?,
                 false => lhs.clone(),
             };
 
