@@ -269,13 +269,13 @@ mod tests {
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(20, 6),
-            vec![
+            [
                 Some(1_234_000),
                 Some(2_222_000),
                 Some(3_000),
                 Some(4_000_000),
                 Some(5_000_000),
-                None,
+                None
             ]
         );
 
@@ -285,7 +285,7 @@ mod tests {
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![Some(123), Some(222), Some(0), Some(400), Some(500), None,]
+            [Some(123), Some(222), Some(0), Some(400), Some(500), None]
         );
 
         Ok(())
@@ -303,13 +303,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int8Array,
             DataType::Int8,
-            vec![
+            [
                 Some(1_i8),
                 Some(2_i8),
                 Some(3_i8),
                 Some(4_i8),
                 Some(5_i8),
-                None,
+                None
             ]
         );
 
@@ -320,13 +320,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int16Array,
             DataType::Int16,
-            vec![
+            [
                 Some(1_i16),
                 Some(2_i16),
                 Some(3_i16),
                 Some(4_i16),
                 Some(5_i16),
-                None,
+                None
             ]
         );
 
@@ -337,13 +337,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int32Array,
             DataType::Int32,
-            vec![
+            [
                 Some(1_i32),
                 Some(2_i32),
                 Some(3_i32),
                 Some(4_i32),
                 Some(5_i32),
-                None,
+                None
             ]
         );
 
@@ -354,13 +354,13 @@ mod tests {
             DataType::Decimal128(10, 0),
             Int64Array,
             DataType::Int64,
-            vec![
+            [
                 Some(1_i64),
                 Some(2_i64),
                 Some(3_i64),
                 Some(4_i64),
                 Some(5_i64),
-                None,
+                None
             ]
         );
 
@@ -372,13 +372,13 @@ mod tests {
             DataType::Decimal128(10, 3),
             Float32Array,
             DataType::Float32,
-            vec![
+            [
                 Some(1.234_f32),
                 Some(2.222_f32),
                 Some(0.003_f32),
                 Some(4.0_f32),
                 Some(5.0_f32),
-                None,
+                None
             ]
         );
         // decimal to float64
@@ -388,13 +388,13 @@ mod tests {
             DataType::Decimal128(20, 6),
             Float64Array,
             DataType::Float64,
-            vec![
+            [
                 Some(0.001234_f64),
                 Some(0.002222_f64),
                 Some(0.000003_f64),
                 Some(0.004_f64),
                 Some(0.005_f64),
-                None,
+                None
             ]
         );
 
@@ -410,7 +410,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(3, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
+            [Some(1), Some(2), Some(3), Some(4), Some(5)]
         );
 
         // int16
@@ -420,7 +420,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(5, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
+            [Some(1), Some(2), Some(3), Some(4), Some(5)]
         );
 
         // int32
@@ -430,7 +430,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(10, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
+            [Some(1), Some(2), Some(3), Some(4), Some(5)]
         );
 
         // int64
@@ -440,7 +440,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 0),
-            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
+            [Some(1), Some(2), Some(3), Some(4), Some(5)]
         );
 
         // int64 to different scale
@@ -450,7 +450,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 2),
-            vec![Some(100), Some(200), Some(300), Some(400), Some(500),]
+            [Some(100), Some(200), Some(300), Some(400), Some(500)]
         );
 
         // float32
@@ -460,7 +460,7 @@ mod tests {
             vec![1.5, 2.5, 3.0, 1.123_456_8, 5.50],
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![Some(150), Some(250), Some(300), Some(112), Some(550),]
+            [Some(150), Some(250), Some(300), Some(112), Some(550)]
         );
 
         // float64
@@ -470,12 +470,12 @@ mod tests {
             vec![1.5, 2.5, 3.0, 1.123_456_8, 5.50],
             Decimal128Array,
             DataType::Decimal128(20, 4),
-            vec![
+            [
                 Some(15000),
                 Some(25000),
                 Some(30000),
                 Some(11235),
-                Some(55000),
+                Some(55000)
             ]
         );
         Ok(())
@@ -489,7 +489,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             UInt32Array,
             DataType::UInt32,
-            vec![
+            [
                 Some(1_u32),
                 Some(2_u32),
                 Some(3_u32),
@@ -508,7 +508,7 @@ mod tests {
             vec![1, 2, 3, 4, 5],
             StringArray,
             DataType::Utf8,
-            vec![Some("1"), Some("2"), Some("3"), Some("4"), Some("5")]
+            [Some("1"), Some("2"), Some("3"), Some("4"), Some("5")]
         );
         Ok(())
     }
@@ -521,7 +521,7 @@ mod tests {
             vec!["a", "2", "3", "b", "5"],
             Int32Array,
             DataType::Int32,
-            vec![None, Some(2), Some(3), None, Some(5)]
+            [None, Some(2), Some(3), None, Some(5)]
         );
         Ok(())
     }
