@@ -16,6 +16,7 @@
 // under the License.
 
 use crate::intervals::Interval;
+use crate::sort_properties::SortProperties;
 use crate::utils::scatter;
 
 use arrow::array::BooleanArray;
@@ -34,7 +35,7 @@ use std::sync::Arc;
 /// Expression that can be evaluated against a RecordBatch
 /// A Physical expression knows its type, nullability and how to evaluate itself.
 pub trait PhysicalExpr: Send + Sync + Display + Debug + PartialEq<dyn Any> {
-    /// Returns the physical expression as [`Any`](std::any::Any) so that it can be
+    /// Returns the physical expression as [`Any`] so that it can be
     /// downcast to a specific implementation.
     fn as_any(&self) -> &dyn Any;
     /// Get the data type of this expression, given the schema of the input
@@ -122,6 +123,19 @@ pub trait PhysicalExpr: Send + Sync + Display + Debug + PartialEq<dyn Any> {
     /// Note: [`PhysicalExpr`] is not constrained by [`Hash`]
     /// directly because it must remain object safe.
     fn dyn_hash(&self, _state: &mut dyn Hasher);
+
+    /// The order information of a PhysicalExpr can be estimated from its children.
+    /// This is especially helpful for projection expressions. If we can ensure that the
+    /// order of a PhysicalExpr to project matches with the order of SortExec, we can
+    /// eliminate that SortExecs.
+    ///
+    /// By recursively calling this function, we can obtain the overall order
+    /// information of the PhysicalExpr. Since `SortOptions` cannot fully handle
+    /// the propagation of unordered columns and literals, the `SortProperties`
+    /// struct is used.
+    fn get_ordering(&self, _children: &[SortProperties]) -> SortProperties {
+        SortProperties::Unordered
+    }
 }
 
 impl Hash for dyn PhysicalExpr {
@@ -167,3 +181,14 @@ pub fn down_cast_any_ref(any: &dyn Any) -> &dyn Any {
         any
     }
 }
+
+/// It is similar to contains method of vector.
+/// Finds whether `expr` is among `physical_exprs`.
+pub fn physical_exprs_contains(
+    physical_exprs: &[Arc<dyn PhysicalExpr>],
+    expr: &Arc<dyn PhysicalExpr>,
+) -> bool {
+    physical_exprs
+        .iter()
+        .any(|physical_expr| physical_expr.eq(expr))
+}
