@@ -31,6 +31,8 @@ use datafusion::logical_expr::window_function::WindowFunction;
 use datafusion::physical_expr::{PhysicalSortExpr, ScalarFunctionExpr};
 use datafusion::physical_plan::expressions::{in_list, LikeExpr};
 use datafusion::physical_plan::expressions::{GetFieldAccessExpr, GetIndexedFieldExpr};
+use datafusion::physical_plan::windows::create_window_expr;
+use datafusion::physical_plan::WindowExpr;
 use datafusion::physical_plan::{
     expressions::{
         BinaryExpr, CaseExpr, CastExpr, Column, IsNotNullExpr, IsNullExpr, Literal,
@@ -84,6 +86,61 @@ pub fn parse_physical_sort_expr(
     }
 }
 
+/// Parses a physical window expr from a protobuf.
+///
+/// # Arguments
+///
+/// * `proto` - Input proto with physical window exprression node.
+/// * `name` - Name of the window expression.
+/// * `registry` - A registry knows how to build logical expressions out of user-defined function' names
+/// * `input_schema` - The Arrow schema for the input, used for determining expression data types
+///                    when performing type coercion.
+pub fn parse_physical_window_expr(
+    proto: &protobuf::PhysicalWindowExprNode,
+    registry: &dyn FunctionRegistry,
+    input_schema: &Schema,
+) -> Result<Arc<dyn WindowExpr>> {
+    let window_node_expr = proto
+        .args
+        .iter()
+        .map(|e| parse_physical_expr(e, registry, input_schema))
+        .collect::<Result<Vec<_>>>()?;
+
+    let partition_by = proto
+        .partition_by
+        .iter()
+        .map(|p| parse_physical_expr(p, registry, input_schema))
+        .collect::<Result<Vec<_>>>()?;
+
+    let order_by = proto
+        .order_by
+        .iter()
+        .map(|o| parse_physical_sort_expr(o, registry, input_schema))
+        .collect::<Result<Vec<_>>>()?;
+
+    let window_frame = proto
+        .window_frame
+        .as_ref()
+        .map(|wf| wf.clone().try_into())
+        .transpose()
+        .map_err(|e| DataFusionError::Internal(format!("{e}")))?
+        .ok_or_else(|| {
+            DataFusionError::Internal(
+                "Missing required field 'window_frame' in protobuf".to_string(),
+            )
+        })?;
+
+    create_window_expr(
+        &convert_required!(proto.window_function)?,
+        proto.name.clone(),
+        &window_node_expr,
+        &partition_by,
+        &order_by,
+        Arc::new(window_frame),
+        input_schema,
+    )
+}
+
 /// Parses a physical expression from a protobuf.
 ///
 /// # Arguments
@@ -230,7 +287,7 @@ pub fn parse_physical_expr(
         )),
         ExprType::ScalarFunction(e) => {
             let scalar_function =
-                protobuf::ScalarFunction::from_i32(e.fun).ok_or_else(|| {
+                protobuf::ScalarFunction::try_from(e.fun).map_err(|_| {
                     proto_error(
                         format!("Received an unknown scalar function: {}", e.fun,),
                     )
@@ -255,6 +312,7 @@ pub fn parse_physical_expr(
                 fun_expr,
                 args,
                 &convert_required!(e.return_type)?,
+                None,
             ))
         }
         ExprType::ScalarUdf(e) => {
@@ -271,6 +329,7 @@ pub fn parse_physical_expr(
                 scalar_fun,
                 args,
                 &convert_required!(e.return_type)?,
+                None,
             ))
         }
         ExprType::LikeExpr(like_expr) => Arc::new(LikeExpr::new(
@@ -356,7 +415,7 @@ impl TryFrom<&protobuf::physical_window_expr_node::WindowFunction> for WindowFun
     ) -> Result<Self, Self::Error> {
         match expr {
             protobuf::physical_window_expr_node::WindowFunction::AggrFunction(n) => {
-                let f = protobuf::AggregateFunction::from_i32(*n).ok_or_else(|| {
+                let f = protobuf::AggregateFunction::try_from(*n).map_err(|_| {
                     proto_error(format!(
                         "Received an unknown window aggregate function: {n}"
                     ))
@@ -365,12 +424,11 @@ impl TryFrom<&protobuf::physical_window_expr_node::WindowFunction> for WindowFun
                 Ok(WindowFunction::AggregateFunction(f.into()))
             }
             protobuf::physical_window_expr_node::WindowFunction::BuiltInFunction(n) => {
-                let f =
-                    protobuf::BuiltInWindowFunction::from_i32(*n).ok_or_else(|| {
-                        proto_error(format!(
-                            "Received an unknown window builtin function: {n}"
-                        ))
-                    })?;
+                let f = protobuf::BuiltInWindowFunction::try_from(*n).map_err(|_| {
+                    proto_error(format!(
+                        "Received an unknown window builtin function: {n}"
+                    ))
+                })?;
 
                 Ok(WindowFunction::BuiltInWindowFunction(f.into()))
             }
